@@ -24,16 +24,17 @@ if 0:
 else:
     export = None
 
-ab = jppy.addressBook()
-_mobile_index = ab.getPhoneLabels().index("Mobile")
-_email_index  = ab.getPhoneLabels().index("E-mail")
-_pers_index   = ab.getPhoneLabels().index("Personal")
+env = jppy.environment.Environment()
+
+_mobile_index = env.addressBook.getPhoneLabels().index("Mobile")
+_email_index  = env.addressBook.getPhoneLabels().index("E-mail")
+_pers_index   = env.addressBook.getPhoneLabels().index("Personal")
 
 for person in email_list.split("\n"):
     if len(person) == 0:
         continue
     parts = person.split(" ")
-    contact = ab.new()
+    contact = env.addressBook.new()
     contact.category = _pers_index
     contact['phone1'] = parts[-1]
     contact['type1']  = _email_index
@@ -47,5 +48,5 @@ for person in email_list.split("\n"):
     contact['custom2'] = "nl-gen-l"
     if export:
         export.write(contact.vcard())
-    ab.save(contact)
+    env.addressBook.save(contact)
     print "Added %s" % contact
