@@ -4,8 +4,9 @@ import jppy
 import vobject
 import datetime
 
-calendar   = jppy.calendar()
-tasks      = jppy.taskList()
+env = jppy.environment.Environment()
+calendar   = env.calendar
+tasks      = env.taskList
 calendar_categories = calendar.getCategories()
 task_categories     = tasks.getCategories()
 
