@@ -5,10 +5,10 @@ from optparse import OptionParser
 import smtplib
 from email.MIMEText import MIMEText
 import datetime
-import pytz
+from dateutil.tz import *
 
-calendar = jppy.calendar()
-calendar.showPrivates = False
+env = jppy.environment.Environment()
+env.calendar.showPrivates = False
 
 parser = OptionParser()
 parser.add_option("-t", "--to",
@@ -35,15 +35,15 @@ if not options.frm:
 
 mail_body = []
 current_day = None
-startOfWeek = datetime.datetime.now(pytz.utc)-datetime.timedelta(
+startOfWeek = datetime.datetime.now(tzutc())-datetime.timedelta(
     days=datetime.datetime.now().weekday())
 endOfPeriod = startOfWeek + datetime.timedelta(days=int(options.period))
 
-# a version of calendar.eventsFromDate which also shows days with no events
-def eventsOnDay(calendar, start, end):
+# a version of env.calendar.eventsFromDate which also shows days with no events
+def eventsOnDay(env, start, end):
     date = start
     while date != end:
-        events = calendar.eventsOnDay(date)
+        events = env.calendar.eventsOnDay(date)
         if len(events) > 0:
             for event in events:
                 yield event
@@ -52,7 +52,7 @@ def eventsOnDay(calendar, start, end):
         date = date + datetime.timedelta(days=1)
             
 
-for date, event in eventsOnDay(calendar, startOfWeek, endOfPeriod):
+for date, event in eventsOnDay(env, startOfWeek, endOfPeriod):
     if date.date() != current_day:
         date_string = date.date().strftime("%A, %d %B %y")
         mail_body.append("\n- %s %s" % (date_string , "-" * (30 - len(date_string))))
