@@ -1,9 +1,15 @@
+#!/usr/bin/python
+
 import jppy
 import mx.DateTime
 
-memos = jppy.memoList()
+env = jppy.environment.Environment()
+memos = env.memoList
 
-cat = memos.getCategories().index("Personal")
+try:
+    cat = memos.getCategories().index("Personal")
+except ValueError, e:
+    cat = 0
 
 fn = "%s.txt" % mx.DateTime.now().date
 
