@@ -7,7 +7,8 @@ parser = OptionParser()
 
 (options, args) = parser.parse_args()
 
-ab = jppy.addressBook()
+env = jppy.environment.Environment()
+ab = env.addressBook
 
 for filename in args:
     vcard = open(filename).read()
@@ -31,6 +32,6 @@ for filename in args:
 
     print vcard
     
-    for contact in jppy.vcard.load_vcards(vcard):
-        ab.save(contact)
+    for contact in ab.loadVCards(vcard):
+        #ab.save(contact)
         print "Added %s" % contact
