@@ -9,7 +9,8 @@ parser = OptionParser()
 
 (options, args) = parser.parse_args()
 
-ab = jppy.addressBook()
+env = jppy.environment.Environment()
+ab = env.addressBook
 
 for filename in args:
     fp = open(filename)
