@@ -1,4 +1,4 @@
-/* $Id: prefs.c,v 1.57 2005/06/24 14:03:13 rousseau Exp $ */
+/* $Id: prefs.c,v 1.86 2009/05/30 16:09:12 rousseau Exp $ */
 
 /*******************************************************************************
  * prefs.c
@@ -20,34 +20,48 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ******************************************************************************/
 
+/********************************* Includes ***********************************/
 #include "config.h"
-#include "i18n.h"
 #include <sys/types.h>
 #include <dirent.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef HAVE_LANGINFO_H
+#  include <langinfo.h>
+#endif
+
+#include "i18n.h"
 #include "utils.h"
 #include "prefs.h"
 #include "log.h"
-#ifdef ENABLE_GTK2
 #include "otherconv.h"
-#endif
 
-/*These are the default settings */
-/*name, usertype, filetype, ivalue, char *svalue, svalue_size; */
+/********************************* Constants **********************************/
+#define NUM_SHORTDATES 7
+#define NUM_LONGDATES 6
+#define NUM_TIMES 10
+#define NUM_TIMES_NO_AMPM 6
+#define NUM_RATES 11
+#define NUM_PAPER_SIZES 2
+
+/******************************* Global vars **********************************/
+int t_fmt_ampm = TRUE;
+
+/* These are the default settings */
+/* name, usertype, filetype, ivalue, char *svalue, svalue_size; */
 static prefType glob_prefs[NUM_PREFS] = {
-     {"rcfile", CHARTYPE, CHARTYPE, 0, NULL, 0},
+     {"jpilotrc", CHARTYPE, CHARTYPE, 0, NULL, 0},
      {"time", CHARTYPE, INTTYPE, 0, NULL, 0},
      {"sdate", CHARTYPE, INTTYPE, 0, NULL, 0},
      {"ldate", CHARTYPE, INTTYPE, 0, NULL, 0},
-     {"fdow", CHARTYPE, INTTYPE, 0, NULL, 0},
+     {"xxx_internal1_xxx", CHARTYPE, INTTYPE, 0, NULL, 0},
      {"show_deleted", INTTYPE, INTTYPE, 0, NULL, 0},
      {"show_modified", INTTYPE, INTTYPE, 0, NULL, 0},
      {"todo_hide_completed", INTTYPE, INTTYPE, 0, NULL, 0},
      {"datebook_highlight_days", INTTYPE, INTTYPE, 1, NULL, 0},
      {"port", CHARTYPE, CHARTYPE, 0, NULL, 0},
-     {"rate", CHARTYPE, INTTYPE, 4, NULL, 0},
+     {"rate", CHARTYPE, INTTYPE, 8, NULL, 0},
      {"user", CHARTYPE, CHARTYPE, 0, NULL, 0},
      {"user_id", INTTYPE, INTTYPE, 0, NULL, 0},
      {"pc_id", INTTYPE, INTTYPE, 0, NULL, 0},
@@ -64,13 +78,7 @@ static prefType glob_prefs[NUM_PREFS] = {
      {"print_one_per_page", INTTYPE, INTTYPE, 0, NULL ,0},
      {"print_blank_lines", INTTYPE, INTTYPE, 1, NULL, 0},
      {"print_command", CHARTYPE, CHARTYPE, 0, NULL, 0},
-     {"char_set", INTTYPE, INTTYPE,
-#ifdef ENABLE_GTK2
-     CHAR_SET_1252_UTF
-#else
-     CHAR_SET_LATIN1
-#endif
-	, NULL, 0},
+     {"char_set", INTTYPE, INTTYPE, CHAR_SET_1252_UTF , NULL, 0},
      {"sync_datebook", INTTYPE, INTTYPE, 1, NULL, 0},
      {"sync_address", INTTYPE, INTTYPE, 1, NULL, 0},
      {"sync_todo", INTTYPE, INTTYPE, 1, NULL, 0},
@@ -81,7 +89,7 @@ static prefType glob_prefs[NUM_PREFS] = {
      {"open_alarm_windows", INTTYPE, INTTYPE, 1, NULL, 0},
      {"do_alarm_command", INTTYPE, INTTYPE, 0, NULL, 0},
      {"alarm_command", CHARTYPE, CHARTYPE, 0, NULL, 0},
-     {"remind_in", CHARTYPE, CHARTYPE, 0, NULL, 0},
+     {"remind_in", INTTYPE, INTTYPE, 0, NULL, 0},
      {"remind_units", INTTYPE, INTTYPE, 0, NULL, 0},
    /* This is actually the password, but I wanted to name it something more discreet */
      {"session_id", CHARTYPE, CHARTYPE, 0, NULL, 0},
@@ -97,7 +105,7 @@ static prefType glob_prefs[NUM_PREFS] = {
      {"memo_import_path", CHARTYPE, CHARTYPE, 0, NULL, 0},
      {"manana_mode", INTTYPE, INTTYPE, 0, NULL, 0},
      {"sync_manana", INTTYPE, INTTYPE, 0, NULL, 0},
-     {"use_jos", INTTYPE, INTTYPE, 0, ""},
+     {"use_jos", INTTYPE, INTTYPE, 0, NULL, 0},
      {"phone_prefix1", CHARTYPE, CHARTYPE, 0, NULL, 0},
      {"phone_prefix1_on", INTTYPE, INTTYPE, 0, NULL, 0},
      {"phone_prefix2", CHARTYPE, CHARTYPE, 0, NULL, 0},
@@ -126,8 +134,23 @@ static prefType glob_prefs[NUM_PREFS] = {
      {"todo_days_till_due", INTTYPE, INTTYPE, 7, NULL, 0},
      {"show_tooltips", INTTYPE, INTTYPE, 1, NULL, 0},
      {"datebook_note_pane", INTTYPE, INTTYPE, 75, NULL, 0},
-     {"datebook_hi_today", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"datebook_hi_today", INTTYPE, INTTYPE, 1, NULL, 0},
      {"datebook_anni_years", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"keyring_pane", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"expense_pane", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"datebook_version", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"address_version", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"todo_version", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"memo_version", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"contacts_photo_filename", CHARTYPE, CHARTYPE, 0, NULL, 0},
+     {"todo_sort_column", INTTYPE, INTTYPE, 1, NULL, 0},
+     {"todo_sort_order", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"addr_sort_order", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"addr_name_col_sz", INTTYPE, INTTYPE, 140, NULL, 0},
+     {"todo_note_pane", INTTYPE, INTTYPE, 200, NULL, 0},
+     {"expense_sort_column", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"expense_sort_order", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"keyr_export_filename", CHARTYPE, CHARTYPE, 0, NULL, 0},
 };
 
 struct name_list {
@@ -137,13 +160,22 @@ struct name_list {
 
 static struct name_list *dir_list=NULL;
 
-
-void pref_init()
+/****************************** Main Code *************************************/
+void pref_init(void)
 {
    int i;
 
+   /* Determine whether locale supports am/pm time formats */
+#  ifdef HAVE_LANGINFO_H
+      t_fmt_ampm = strcmp(nl_langinfo(T_FMT_AMPM), "");
+#  endif
+
    for (i=0; i<NUM_PREFS; i++) {
       switch (i) {
+       case PREF_PORT:
+	 glob_prefs[i].svalue=strdup("usb:");
+	 glob_prefs[i].svalue_size=strlen(glob_prefs[i].svalue)+1;
+         break;
        case PREF_RCFILE:
 	 glob_prefs[i].svalue=strdup(EPN"rc.default");
 	 glob_prefs[i].svalue_size=strlen(glob_prefs[i].svalue)+1;
@@ -169,7 +201,7 @@ void pref_init()
 	 glob_prefs[i].svalue_size=strlen(glob_prefs[i].svalue)+1;
 	 break;
        case PREF_MAIL_COMMAND:
-	 glob_prefs[i].svalue=strdup("mozilla -remote \"mailto(\%s)\"");
+	 glob_prefs[i].svalue=strdup("mozilla-thunderbird -remote \"mailto(%s)\"");
 	 glob_prefs[i].svalue_size=strlen(glob_prefs[i].svalue)+1;
 	 break;
        default:
@@ -268,7 +300,7 @@ int get_pref_time_no_secs_no_ampm(char *datef)
    return EXIT_SUCCESS;
 }
 
-/*This function is used internally to free up any memory that prefs is using */
+/* This function is used internally to free up any memory that prefs is using */
 /* I'm not using this function right now.
 void free_name_list(struct name_list **Plist)
 {
@@ -284,6 +316,7 @@ void free_name_list(struct name_list **Plist)
    *Plist=NULL;
 }
 */
+
 static int get_rcfile_name(int n, char *rc_copy)
 {
    DIR *dir;
@@ -351,8 +384,7 @@ static int get_rcfile_name(int n, char *rc_copy)
    found = 0;
    for (i=0, temp_list=dir_list; temp_list; temp_list=temp_list->next, i++) {
       if (i == n) {
-	 strncpy(rc_copy, temp_list->name, MAX_PREF_VALUE);
-	 rc_copy[MAX_PREF_VALUE-1]='\0';
+	 g_strlcpy(rc_copy, temp_list->name, MAX_PREF_LEN);
 	 found=1;
 	 break;
       }
@@ -366,13 +398,7 @@ static int get_rcfile_name(int n, char *rc_copy)
    }
 }
 
-#define NUM_SHORTDATES 7
-#define NUM_LONGDATES 6
-#define NUM_TIMES 10
-#define NUM_RATES 11
-#define NUM_PAPER_SIZES 2
-
-/*if n is out of range then this function will fail */
+/* if n is out of range then this function will fail */
 int get_pref_possibility(int which, int n, char *pref_str)
 {
    const char *short_date_formats[] = {
@@ -407,6 +433,15 @@ int get_pref_possibility(int which, int n, char *pref_str)
       "%H,%M"
    };
 
+   const char *time_formats_no_ampm[] = {
+      "%H:%M:%S",
+      "%H.%M.%S",
+      "%H,%M,%S",
+      "%H:%M",
+      "%H.%M",
+      "%H,%M"
+   };
+
    static char *days[] = {
       N_("Sunday"),
       N_("Monday")
@@ -434,16 +469,17 @@ int get_pref_possibility(int which, int n, char *pref_str)
       "Host KOI8-R <-> Palm Windows-1251",
       "Chinese(Big5)",
       "Korean",
-      "UTF: Latin 2, Estern Europe (CP1250)",
+      "UTF: Latin 2, Eastern Europe (CP1250)",
       "UTF: Latin 1, Western Europe (CP1252)",
       "UTF: Greek (CP1253)",
-      "UTF: Latin 2, Estern Europe (ISO8859-2)",
+      "UTF: Latin 2, Eastern Europe (ISO8859-2)",
       "UTF: Cyrillic (KOI8-R)",
       "UTF: Cyrillic (CP1251)",
       "UTF: Simplified Chinese (GBK)",
       "UTF: Japanese (SJIS)",
       "UTF: Hebrew (CP1255)",
       "UTF: Traditional Chinese (BIG-5)",
+      "UTF: Korean (CP949)",
    };
 
    static const char *paper_sizes[] = {
@@ -451,19 +487,27 @@ int get_pref_possibility(int which, int n, char *pref_str)
       "A4"
    };
 
-   switch(which) {
+   switch (which) {
 
     case PREF_RCFILE:
 	 return get_rcfile_name(n, pref_str);
       break;
 
     case PREF_TIME:
-      if ((n >= NUM_TIMES) || (n<0)) {
-	 pref_str[0]='\0';
-	 return EXIT_FAILURE;
+      if (t_fmt_ampm) {
+	 if ((n >= NUM_TIMES) || (n<0)) {
+	    pref_str[0]='\0';
+	    return EXIT_FAILURE;
+	 }
+	 strcpy(pref_str, time_formats[n]);
+      } else {
+	 if ((n >= NUM_TIMES_NO_AMPM) || (n<0)) {
+	    pref_str[0]='\0';
+	    return EXIT_FAILURE;
+	 }
+	 strcpy(pref_str, time_formats_no_ampm[n]);
       }
-      strcpy(pref_str, time_formats[n]);
-      break;
+	 break;
 
     case PREF_SHORTDATE:
       if ((n >= NUM_SHORTDATES) || (n<0)) {
@@ -522,10 +566,7 @@ int get_pref_possibility(int which, int n, char *pref_str)
    return EXIT_SUCCESS;
 }
 
-/*
- * WARNING
- * Caller must ensure that which is not out of range!
- */
+/* Warning: which must be in the range of choices or this can seg fault */
 int jp_get_pref(prefType prefs[], int which, long *n, const char **string)
 {
    if (which < 0) {
@@ -534,21 +575,20 @@ int jp_get_pref(prefType prefs[], int which, long *n, const char **string)
    if (n) {
       *n = prefs[which].ivalue;
    }
-   if (prefs[which].usertype == CHARTYPE) {
-      if (string!=NULL) {
+   if (string!=NULL) {
+      if (prefs[which].usertype == CHARTYPE) {
 	 *string = prefs[which].svalue;
-      }
-   } else {
-      if (string!=NULL) {
+      } else {
 	 *string = NULL;
       }
    }
+
    return EXIT_SUCCESS;
 }
 
 int get_pref(int which, long *n, const char **string)
 {
-   if (which > NUM_PREFS) {
+   if (which >= NUM_PREFS) {
       return EXIT_FAILURE;
    }
    return jp_get_pref(glob_prefs, which, n, string);
@@ -603,8 +643,7 @@ char *pref_lstrncpy_realloc(char **dest, const char *src, int *size, int max_siz
       }
       *size=new_size;
    }
-   strncpy(*dest, Psrc, new_size);
-   (*dest)[new_size-1]='\0';
+   g_strlcpy(*dest, Psrc, new_size);
 
    return *dest;
 }
@@ -625,7 +664,7 @@ int jp_set_pref(prefType prefs[], int which, long n, const char *string)
    }
    if (prefs[which].usertype == CHARTYPE) {
       pref_lstrncpy_realloc(&(prefs[which].svalue), Pstr,
-			    &(prefs[which].svalue_size), MAX_PREF_VALUE);
+			    &(prefs[which].svalue_size), MAX_PREF_LEN);
    }
    return EXIT_SUCCESS;
 }
@@ -635,7 +674,7 @@ int set_pref(int which, long n, const char *string, int save)
    const char *str;
    int r;
 
-   if (which > NUM_PREFS) {
+   if (which >= NUM_PREFS) {
       return EXIT_FAILURE;
    }
    str=string;
@@ -648,23 +687,20 @@ int set_pref(int which, long n, const char *string, int save)
       str=glob_prefs[which].svalue;
    }
    r = jp_set_pref(glob_prefs, which, n, str);
-   /* #ifdef ENABLE_PROMETHEON */
-   /* Some people like to just kill the window manager */
-   /* Prometheon kills us, always be prepared for death */
    if (save) {
       pref_write_rc_file();
    }
-   /* #endif */
+
    return r;
 }
 
 int set_pref_possibility(int which, long n, int save)
 {
-   char svalue[MAX_PREF_VALUE];
+   char svalue[MAX_PREF_LEN];
    char *str=NULL;
    int r;
 
-   if (which > NUM_PREFS) {
+   if (which >= NUM_PREFS) {
       return EXIT_FAILURE;
    }
    if (glob_prefs[which].usertype == CHARTYPE) {
@@ -672,30 +708,31 @@ int set_pref_possibility(int which, long n, int save)
       str=svalue;
    }
    r = jp_set_pref(glob_prefs, which, n, str);
-   /* #ifdef ENABLE_PROMETHEON */
-   /* Some people like to just kill the window manager */
-   /* Prometheon kills us, always be prepared for death */
    if (save) {
       pref_write_rc_file();
    }
-   /* #endif */
 
-#ifdef ENABLE_GTK2
    if (PREF_CHAR_SET == which)
       if (otherconv_init())
-	 printf("Error: could not set encoding\n");
-#endif
+	 printf("Error: could not set charset encoding\n");
 
    return r;
 }
 
-static int validate_glob_prefs()
+static int validate_glob_prefs(void)
 {
    int i, r;
-   char svalue[MAX_PREF_VALUE];
+   char svalue[MAX_PREF_LEN];
 
-   if (glob_prefs[PREF_TIME].ivalue >= NUM_TIMES) {
-      glob_prefs[PREF_TIME].ivalue = NUM_TIMES - 1;
+   if (t_fmt_ampm) {
+      if (glob_prefs[PREF_TIME].ivalue >= NUM_TIMES) {
+	 glob_prefs[PREF_TIME].ivalue = NUM_TIMES - 1;
+      }
+   }
+   else {
+      if (glob_prefs[PREF_TIME].ivalue >= NUM_TIMES_NO_AMPM) {
+	 glob_prefs[PREF_TIME].ivalue = NUM_TIMES_NO_AMPM - 1;
+      }
    }
    if (glob_prefs[PREF_TIME].ivalue < 0) {
       glob_prefs[PREF_TIME].ivalue = 0;
@@ -778,32 +815,42 @@ static int validate_glob_prefs()
       glob_prefs[PREF_NUM_BACKUPS].ivalue = 1;
    }
 
+   /* Backwards compatability with MEMO32 preference */
+   if (glob_prefs[PREF_MEMO32_MODE].ivalue) {
+      glob_prefs[PREF_MEMO_VERSION].ivalue = 2;
+      glob_prefs[PREF_MEMO32_MODE].ivalue = 0;
+   }
+   if (glob_prefs[PREF_SYNC_MEMO32].ivalue) {
+      glob_prefs[PREF_SYNC_MEMO].ivalue = 1;
+      glob_prefs[PREF_SYNC_MEMO32].ivalue = 0;
+   }
+
    get_pref_possibility(PREF_TIME, glob_prefs[PREF_TIME].ivalue, svalue);
    pref_lstrncpy_realloc(&(glob_prefs[PREF_TIME].svalue), svalue,
-			 &(glob_prefs[PREF_TIME].svalue_size), MAX_PREF_VALUE);
+			 &(glob_prefs[PREF_TIME].svalue_size), MAX_PREF_LEN);
 
    get_pref_possibility(PREF_SHORTDATE, glob_prefs[PREF_SHORTDATE].ivalue, svalue);
    pref_lstrncpy_realloc(&(glob_prefs[PREF_SHORTDATE].svalue), svalue,
-			 &(glob_prefs[PREF_SHORTDATE].svalue_size), MAX_PREF_VALUE);
+			 &(glob_prefs[PREF_SHORTDATE].svalue_size), MAX_PREF_LEN);
 
 
    get_pref_possibility(PREF_LONGDATE, glob_prefs[PREF_LONGDATE].ivalue, svalue);
    pref_lstrncpy_realloc(&(glob_prefs[PREF_LONGDATE].svalue), svalue,
-			 &(glob_prefs[PREF_LONGDATE].svalue_size), MAX_PREF_VALUE);
+			 &(glob_prefs[PREF_LONGDATE].svalue_size), MAX_PREF_LEN);
 
    get_pref_possibility(PREF_FDOW, glob_prefs[PREF_FDOW].ivalue, svalue);
    pref_lstrncpy_realloc(&(glob_prefs[PREF_FDOW].svalue), svalue,
-			 &(glob_prefs[PREF_FDOW].svalue_size), MAX_PREF_VALUE);
+			 &(glob_prefs[PREF_FDOW].svalue_size), MAX_PREF_LEN);
 
    get_pref_possibility(PREF_RATE, glob_prefs[PREF_RATE].ivalue, svalue);
    pref_lstrncpy_realloc(&(glob_prefs[PREF_RATE].svalue), svalue,
-			 &(glob_prefs[PREF_RATE].svalue_size), MAX_PREF_VALUE);
+			 &(glob_prefs[PREF_RATE].svalue_size), MAX_PREF_LEN);
 
    get_pref_possibility(PREF_PAPER_SIZE, glob_prefs[PREF_PAPER_SIZE].ivalue, svalue);
    pref_lstrncpy_realloc(&(glob_prefs[PREF_PAPER_SIZE].svalue), svalue,
-			 &(glob_prefs[PREF_PAPER_SIZE].svalue_size), MAX_PREF_VALUE);
+			 &(glob_prefs[PREF_PAPER_SIZE].svalue_size), MAX_PREF_LEN);
 
-   for (i=0; i<1000; i++) {
+   for (i=0; i<MAX_NUM_PREFS; i++) {
       r = get_pref_possibility(PREF_RCFILE, i, svalue);
       if (r) break;
       if (!strcmp(svalue, glob_prefs[PREF_RCFILE].svalue)) {
@@ -811,6 +858,7 @@ static int validate_glob_prefs()
 	 break;
       }
    }
+
    return EXIT_SUCCESS;
 }
 
@@ -833,7 +881,7 @@ int jp_pref_read_rc_file(char *filename, prefType prefs[], int num_prefs)
       line[sizeof(line)-2] = ' ';
       line[sizeof(line)-1] = '\0';
       field1 = strtok(line, " ");
-      field2 = (field1 != NULL)	? strtok(NULL, "\n") : NULL;/* jonh */
+      field2 = (field1 != NULL)	? strtok(NULL, "\n") : NULL;
       if ((field1 == NULL) || (field2 == NULL)) {
 	 continue;
       }
@@ -848,7 +896,7 @@ int jp_pref_read_rc_file(char *filename, prefType prefs[], int num_prefs)
 	    if (prefs[i].filetype == CHARTYPE) {
 	       if (pref_lstrncpy_realloc(&(prefs[i].svalue), field2,
 					&(prefs[i].svalue_size),
-					MAX_PREF_VALUE)==NULL) {
+					MAX_PREF_LEN)==NULL) {
 		  jp_logf(JP_LOG_WARN, "read_rc_file(): %s\n", _("Out of memory"));
 		  continue;
 	       }
@@ -861,11 +909,11 @@ int jp_pref_read_rc_file(char *filename, prefType prefs[], int num_prefs)
    return EXIT_SUCCESS;
 }
 
-int pref_read_rc_file()
+int pref_read_rc_file(void)
 {
    int r;
 
-   r=jp_pref_read_rc_file(EPN".rc", glob_prefs, NUM_PREFS);
+   r = jp_pref_read_rc_file(EPN".rc", glob_prefs, NUM_PREFS);
 
    validate_glob_prefs();
 
@@ -894,12 +942,13 @@ int jp_pref_write_rc_file(char *filename, prefType prefs[], int num_prefs)
 	 fprintf(out, "%s %s\n", prefs[i].name, prefs[i].svalue);
       }
    }
-   fclose(out);
+   jp_close_home_file(out);
 
    return EXIT_SUCCESS;
 }
 
-int pref_write_rc_file()
+int pref_write_rc_file(void)
 {
    return jp_pref_write_rc_file(EPN".rc", glob_prefs, NUM_PREFS);
 }
+
