@@ -1,4 +1,4 @@
-/* $Id: prefs_gui.c,v 1.46 2005/06/24 14:03:13 rousseau Exp $ */
+/* $Id: prefs_gui.c,v 1.68 2008/06/19 04:12:07 rikster5 Exp $ */
 
 /*******************************************************************************
  * prefs_gui.c
@@ -20,17 +20,20 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ******************************************************************************/
 
+/********************************* Includes ***********************************/
 #include "config.h"
-#include "i18n.h"
-#include <gtk/gtk.h>
 #include <stdlib.h>
 #include <string.h>
-#include "utils.h"
-#include "prefs.h"
+#include <gtk/gtk.h>
+
 #include "prefs_gui.h"
+#include "prefs.h"
+#include "i18n.h"
+#include "utils.h"
 #include "log.h"
 #include "plugins.h"
 
+/******************************* Global vars **********************************/
 static GtkWidget *window;
 static GtkWidget *main_window;
 static GtkWidget *port_entry;
@@ -41,7 +44,21 @@ static GtkWidget *todo_days_due_entry;
 
 extern int glob_app;
 extern GtkTooltips *glob_tooltips;
+#ifdef ENABLE_PLUGINS
+extern unsigned char skip_plugins;
+#endif
 
+/* Serial Port Menu */
+static GtkWidget *port_menu;
+GtkWidget *port_menu_item[10];
+static char *port_choices[]={
+   "other", "usb:",
+     "/dev/ttyUSB0", "/dev/ttyUSB1", "/dev/ttyUSB2", "/dev/ttyUSB3",
+     "/dev/ttyS0", "/dev/ttyS1", "/dev/ttyS2", "/dev/ttyS3",
+     NULL
+};
+
+/****************************** Main Code *************************************/
 #ifdef COLORS
 
 /* This doesn't work quite right.  There is supposedly no way to do it in GTK. */
@@ -73,10 +90,66 @@ void set_colors()
    gtk_widget_queue_draw(window);
    gtk_widget_queue_draw(main_window);
 }
-#endif
+#endif /* #ifdef COLORS */
 
-static void cb_pref_menu(GtkWidget *widget,
-			 gpointer   data)
+/* Serial Port menu code */
+static void cb_serial_port_menu(GtkWidget *widget,
+				gpointer   data)
+{
+   if (!widget)
+     return;
+   if (!(GTK_CHECK_MENU_ITEM(widget))->active) {
+      return;
+   }
+
+   gtk_entry_set_text(GTK_ENTRY(port_entry), port_choices[GPOINTER_TO_INT(data)]);
+
+   return;
+}
+
+int make_serial_port_menu(GtkWidget **port_menu)
+{
+   GtkWidget *menu;
+   GSList    *group;
+   int i, selected;
+   const char *entry_text;
+
+   *port_menu = gtk_option_menu_new();
+
+   menu = gtk_menu_new();
+   group = NULL;
+   selected=0;
+
+   entry_text = gtk_entry_get_text(GTK_ENTRY(port_entry));
+
+   for (i=0; port_choices[i]; i++) {
+      port_menu_item[i] = gtk_radio_menu_item_new_with_label(group, port_choices[i]);
+      group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(port_menu_item[i]));
+      gtk_menu_append(GTK_MENU(menu), port_menu_item[i]);
+
+      if (!strcmp(entry_text, port_choices[i])) {
+	 gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(port_menu_item[i]), TRUE);
+	 selected=i;
+      }
+
+      /* We don't want a callback if "other" is selected */
+      if (i) {
+	 gtk_signal_connect(GTK_OBJECT(port_menu_item[i]), "activate", GTK_SIGNAL_FUNC(cb_serial_port_menu),
+			    GINT_TO_POINTER(i));
+      }
+
+      gtk_widget_show(port_menu_item[i]);
+   }
+   gtk_option_menu_set_menu(GTK_OPTION_MENU(*port_menu), menu);
+
+   gtk_option_menu_set_history(GTK_OPTION_MENU(*port_menu), selected);
+
+   return EXIT_SUCCESS;
+}
+
+/* End Serial Port Menu code */
+
+static void cb_pref_menu(GtkWidget *widget, gpointer data)
 {
    int pref;
    int value;
@@ -108,8 +181,8 @@ int make_pref_menu(GtkWidget **pref_menu, int pref_num)
    int i, r;
    long ivalue;
    const char *svalue;
-   char format_text[MAX_PREF_VALUE];
-   char human_text[MAX_PREF_VALUE];
+   char format_text[MAX_PREF_LEN];
+   char human_text[MAX_PREF_LEN];
    time_t ltime;
    struct tm *now;
 
@@ -123,7 +196,7 @@ int make_pref_menu(GtkWidget **pref_menu, int pref_num)
 
    get_pref(pref_num, &ivalue, &svalue);
 
-   for (i=0; i<1000; i++) {
+   for (i=0; i<MAX_NUM_PREFS; i++) {
       r = get_pref_possibility(pref_num, i, format_text);
       if (r) {
 	 break;
@@ -131,13 +204,13 @@ int make_pref_menu(GtkWidget **pref_menu, int pref_num)
       switch (pref_num) {
        case PREF_SHORTDATE:
        case PREF_TIME:
-	 jp_strftime(human_text, MAX_PREF_VALUE, format_text, now);
+	 jp_strftime(human_text, MAX_PREF_LEN, format_text, now);
 	 break;
        case PREF_LONGDATE:
-	 jp_strftime(human_text, MAX_PREF_VALUE, _(format_text), now);
+	 jp_strftime(human_text, MAX_PREF_LEN, _(format_text), now);
 	 break;
        default:
-	 strncpy(human_text, format_text, MAX_PREF_VALUE);
+	 strncpy(human_text, format_text, MAX_PREF_LEN);
 	 break;
       }
       menu_item = gtk_radio_menu_item_new_with_label(
@@ -202,19 +275,55 @@ void cb_checkbox_show_tooltips(GtkWidget *widget, gpointer data)
 void cb_text_entry(GtkWidget *widget, gpointer data)
 {
    const char *entry_text;
+   int i, found;
 
    entry_text = gtk_entry_get_text(GTK_ENTRY(widget));
    set_pref(GPOINTER_TO_INT(data), 0, entry_text, FALSE);
+
+   if (GPOINTER_TO_INT(data) == PREF_PORT) {
+      if (GTK_IS_WIDGET(port_menu_item[0])) {
+	 found=0;
+	 for (i=0; port_choices[i]; i++) {
+	    if (!strcmp(entry_text, port_choices[i])) {
+	       gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(port_menu_item[i]), TRUE);
+	       gtk_option_menu_set_history(GTK_OPTION_MENU(port_menu), i);
+	       found=1;
+	    }
+	 }
+	 if (!found) {
+	    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(port_menu_item[0]), TRUE);
+	    gtk_option_menu_set_history(GTK_OPTION_MENU(port_menu), 0);
+	 }
+      }
+   }
 }
 
 void cb_checkbox_set_pref(GtkWidget *widget, gpointer data)
 {
-   set_pref(GPOINTER_TO_INT(data), GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
+   unsigned long pref, value;
+
+   pref = GPOINTER_TO_INT(data);
+   value = GTK_TOGGLE_BUTTON(widget)->active;
+   set_pref(pref, value, NULL, TRUE);
 }
 
+/*
+ * upper 16 bits of data is pref to set
+ * lower 16 bits of data is value to set it to
+ */
+void cb_radio_set_pref(GtkWidget *widget, gpointer data)
+{
+   unsigned long pref, value;
+
+   pref=GPOINTER_TO_INT(data);
+   value=pref & 0xFFFF;
+   pref >>= 16;
+   set_pref(pref, value, NULL, TRUE);
+}
+
+
 #ifdef ENABLE_PLUGINS
-void cb_sync_plugin(GtkWidget *widget,
-		    gpointer data)
+void cb_sync_plugin(GtkWidget *widget, gpointer data)
 {
    GList *plugin_list, *temp_list;
    struct plugin_s *Pplugin;
@@ -244,7 +353,7 @@ void cb_sync_plugin(GtkWidget *widget,
 
 static gboolean cb_destroy(GtkWidget *widget)
 {
-   jp_logf(JP_LOG_DEBUG, "Cleanup\n");
+   jp_logf(JP_LOG_DEBUG, "Pref GUI Cleanup\n");
 
    pref_write_rc_file();
 
@@ -266,11 +375,11 @@ static void cb_quit(GtkWidget *widget, gpointer data)
 }
 
 /* This function adds a simple option checkbutton for the supplied text +
- * option.
- */
-static void
-add_checkbutton(const char *text, int which, GtkWidget *vbox,
-		void cb(GtkWidget *widget, gpointer data))
+ * option.  */
+static void add_checkbutton(const char *text, 
+                            int which, 
+                            GtkWidget *vbox, 
+                            void cb(GtkWidget *widget, gpointer data))
 {
    /* Create button */
    GtkWidget *checkbutton = gtk_check_button_new_with_label(text);
@@ -303,11 +412,20 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    GtkWidget *vbox_alarms;
    GtkWidget *vbox_conduits;
    GtkWidget *hbox_temp;
+   GtkWidget *hseparator;
    GtkWidget *notebook;
+   /* FIXME: Uncomment when support for Calendar and Task has been added */
+#if 0
+   GtkWidget *radio_button_datebook_version[2];
+   GtkWidget *radio_button_todo_version[2];
+#endif
+   GtkWidget *radio_button_address_version[2];
+   GtkWidget *radio_button_memo_version[3];
    long ivalue;
    const char *cstr;
    char temp_str[10];
    char temp[256];
+   GSList *group;
 #ifdef ENABLE_PLUGINS
    GList *plugin_list, *temp_list;
    struct plugin_s *Pplugin;
@@ -317,19 +435,15 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    jp_logf(JP_LOG_DEBUG, "cb_prefs_gui\n");
    if (window) {
       jp_logf(JP_LOG_DEBUG, "pref_window is already up\n");
-#ifdef ENABLE_GTK2
       /* Shift focus to existing window if called again
          and window is still alive. */
       gtk_window_present(GTK_WINDOW(window));
-#endif
       return;
    }
 
    main_window = data;
 
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-   /*gtk_window_set_default_size(GTK_WINDOW(window), 500, 300); */
-
    gtk_container_set_border_width(GTK_CONTAINER(window), 10);
    g_snprintf(temp, sizeof(temp), "%s %s", PN, _("Preferences"));
    gtk_window_set_title(GTK_WINDOW(window), temp);
@@ -340,6 +454,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    vbox = gtk_vbox_new(FALSE, 5);
    gtk_container_add(GTK_CONTAINER(window), vbox);
 
+   /* Boxes for each preference tax */
    vbox_locale   = gtk_vbox_new(FALSE, 0);
    vbox_settings = gtk_vbox_new(FALSE, 0);
    vbox_datebook = gtk_vbox_new(FALSE, 0);
@@ -358,7 +473,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_container_set_border_width(GTK_CONTAINER(vbox_alarms), 5);
    gtk_container_set_border_width(GTK_CONTAINER(vbox_conduits), 5);
 
-   /*Add the notebook for repeat types */
+   /* Notebook for preference tabs */
    notebook = gtk_notebook_new();
    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
    label = gtk_label_new(_("Locale"));
@@ -367,7 +482,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_settings, label);
    label = gtk_label_new(_("Datebook"));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_datebook, label);
-   label = gtk_label_new(_("Address"));
+   label = gtk_label_new(C_("prefgui","Address"));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_address, label);
    label = gtk_label_new(_("ToDo"));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_todo, label);
@@ -383,11 +498,11 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    /* Locale preference tab */
    table = gtk_table_new(5, 2, FALSE);
    gtk_table_set_row_spacings(GTK_TABLE(table),0);
-   gtk_table_set_col_spacings(GTK_TABLE(table),0);
+   gtk_table_set_col_spacings(GTK_TABLE(table),5);
    gtk_box_pack_start(GTK_BOX(vbox_locale), table, FALSE, FALSE, 0);
 
    /* Character Set */
-   label = gtk_label_new(_("Character Set "));
+   label = gtk_label_new(_("Character Set"));
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(label),
 			     0, 1, 0, 1);
    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
@@ -400,7 +515,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
 
    /* Shortdate */
-   label = gtk_label_new(_("Short date format "));
+   label = gtk_label_new(_("Short date format"));
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(label),
 			     0, 1, 1, 2);
    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
@@ -413,7 +528,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
 
    /* Longdate */
-   label = gtk_label_new(_("Long date format "));
+   label = gtk_label_new(_("Long date format"));
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(label),
 			     0, 1, 2, 3);
    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
@@ -426,7 +541,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
 
    /* Time */
-   label = gtk_label_new(_("Time format "));
+   label = gtk_label_new(_("Time format"));
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(label),
 			     0, 1, 3, 4);
    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
@@ -438,50 +553,36 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    get_pref(PREF_TIME, &ivalue, &cstr);
    gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
 
-   /* FDOW */
-#  ifndef ENABLE_GTK2
-   label = gtk_label_new(_("The first day of the week is "));
-   gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(label),
-			     0, 1, 4, 5);
-   gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-
-   make_pref_menu(&pref_menu, PREF_FDOW);
-   gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(pref_menu),
-			     1, 2, 4, 5);
-
-   get_pref(PREF_FDOW, &ivalue, &cstr);
-   gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
-#  endif
-
    /**********************************************************************/
    /* Settings preference tab */
-   table = gtk_table_new(4, 2, FALSE);
+   table = gtk_table_new(4, 3, FALSE);
    gtk_table_set_row_spacings(GTK_TABLE(table),0);
-   gtk_table_set_col_spacings(GTK_TABLE(table),0);
+   gtk_table_set_col_spacings(GTK_TABLE(table),5);
    gtk_box_pack_start(GTK_BOX(vbox_settings), table, FALSE, FALSE, 0);
 
    /* GTK colors file */
-   label = gtk_label_new(_("My GTK colors file is "));
+   label = gtk_label_new(_("GTK color theme file"));
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(label),
-			     0, 1, 0, 1);
+			     0, 2, 0, 1);
    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
 
    make_pref_menu(&pref_menu, PREF_RCFILE);
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(pref_menu),
-			     1, 2, 0, 1);
+			     2, 3, 0, 1);
 
    get_pref(PREF_RCFILE, &ivalue, &cstr);
    gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
 
    /* Port */
-   label = gtk_label_new(_("Serial Port (/dev/ttyS0, /dev/pilot)"));
+   label = gtk_label_new(_("Serial Port"));
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(label),
 			     0, 1, 1, 2);
    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
 
-   port_entry = gtk_entry_new_with_max_length(MAX_PREF_VALUE - 2);
+   port_entry = gtk_entry_new_with_max_length(MAX_PREF_LEN - 2);
+   entry_set_multiline_truncate(GTK_ENTRY(port_entry), TRUE);
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(port_entry),
-			     1, 2, 1, 2);
+			     2, 3, 1, 2);
    get_pref(PREF_PORT, &ivalue, &cstr);
    if (cstr) {
       gtk_entry_set_text(GTK_ENTRY(port_entry), cstr);
@@ -490,15 +591,21 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
 		      "changed", GTK_SIGNAL_FUNC(cb_text_entry),
 		      GINT_TO_POINTER(PREF_PORT));
 
-   /* Rate */
-   label = gtk_label_new(_("Serial Rate (Does not affect USB)"));
+   /* Serial Port Menu */
+   /* Note that port_entry must exist before we call this function */
+   make_serial_port_menu(&port_menu);
+   gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(port_menu),
+			     1, 2, 1, 2);
+
+   /* Serial Rate */
+   label = gtk_label_new(_("Serial Rate"));
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(label),
-			     0, 1, 2, 3);
+			     0, 2, 2, 3);
    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
 
    make_pref_menu(&pref_menu, PREF_RATE);
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(pref_menu),
-			     1, 2, 2, 3);
+			     2, 3, 2, 3);
 
    get_pref(PREF_RATE, &ivalue, &cstr);
    gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
@@ -506,13 +613,14 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    /* Number of backups */
    label = gtk_label_new(_("Number of backups to be archived"));
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(label),
-			     0, 1, 3, 4);
+			     0, 2, 3, 4);
    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
 
    backups_entry = gtk_entry_new_with_max_length(2);
+   entry_set_multiline_truncate(GTK_ENTRY(backups_entry), TRUE);
    gtk_widget_set_usize(backups_entry, 30, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(backups_entry),
-			     1, 2, 3, 4);
+			     2, 3, 3, 4);
    get_pref(PREF_NUM_BACKUPS, &ivalue, &cstr);
    sprintf(temp_str, "%ld", ivalue);
    gtk_entry_set_text(GTK_ENTRY(backups_entry), temp_str);
@@ -541,35 +649,99 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    /**********************************************************************/
    /* Datebook preference tab */
 
+   /* FIXME: undef when support for Calendar has been coded */
+#if 0
+   /* Radio box to choose which database to use: Datebook/Calendar */
+   group = NULL;
+   radio_button_datebook_version[0] = 
+     gtk_radio_button_new_with_label(group, _("Use Datebook database (Palm OS < 5.2.1)"));
+   group = gtk_radio_button_group(GTK_RADIO_BUTTON(radio_button_datebook_version[0]));
+   radio_button_datebook_version[1] = 
+     gtk_radio_button_new_with_label(group, _("Use Calendar database (Palm OS > 5.2)"));
+   group = gtk_radio_button_group(GTK_RADIO_BUTTON(radio_button_datebook_version[1]));
+   gtk_box_pack_start(GTK_BOX(vbox_datebook), radio_button_datebook_version[0],
+		      FALSE, FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(vbox_datebook), radio_button_datebook_version[1],
+		      FALSE, FALSE, 0);
+
+   gtk_signal_connect(GTK_OBJECT(radio_button_datebook_version[0]), "pressed",
+		      GTK_SIGNAL_FUNC(cb_radio_set_pref),
+		      GINT_TO_POINTER((PREF_DATEBOOK_VERSION<<16)|0));
+   gtk_signal_connect(GTK_OBJECT(radio_button_datebook_version[1]), "pressed",
+		      GTK_SIGNAL_FUNC(cb_radio_set_pref),
+		      GINT_TO_POINTER((PREF_DATEBOOK_VERSION<<16)|1));
+
+   get_pref(PREF_DATEBOOK_VERSION, &ivalue, NULL);
+   if (ivalue) {
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button_datebook_version[1]), TRUE);
+   } else {
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button_datebook_version[0]), TRUE);
+   }
+
+   /* Separate database selection from less important options */
+   hseparator = gtk_hseparator_new();
+   gtk_box_pack_start(GTK_BOX(vbox_address), hseparator, FALSE, FALSE, 3);
+#endif
+
    /* Show highlight days check box */
    add_checkbutton(_("Highlight calendar days with appointments"),
 		   PREF_DATEBOOK_HIGHLIGHT_DAYS, vbox_datebook,
 		   cb_checkbox_set_pref);
 
+   /* Highlight today on month and week view */
+   add_checkbutton(_("Annotate today in day, week, and month views"),
+ 		   PREF_DATEBOOK_HI_TODAY, vbox_datebook, cb_checkbox_set_pref);
+ 
+   /* Show number of years on anniversaries in month and week view */
+   add_checkbutton(_("Append years on anniversaries in day, week, and month views"),
+ 	           PREF_DATEBOOK_ANNI_YEARS, vbox_datebook,
+ 	           cb_checkbox_set_pref);
+
 #ifdef ENABLE_DATEBK
    /* Show use DateBk check box */
    add_checkbutton(_("Use DateBk note tags"),
 		   PREF_USE_DB3, vbox_datebook, cb_checkbox_set_pref);
 #else
-   label = gtk_label_new(_("DateBk support disabled in this build"));
-   gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-   gtk_box_pack_start(GTK_BOX(vbox_datebook), label, FALSE, FALSE, 0);
+   checkbutton = gtk_check_button_new_with_label(_("DateBk support disabled in this build"));
+   gtk_widget_set_sensitive(checkbutton, FALSE);
+   gtk_box_pack_start(GTK_BOX(vbox_datebook), checkbutton, FALSE, FALSE, 0);
+   gtk_widget_show(checkbutton);
 #endif
-
-    /* Highlight today on month and week view */
-    add_checkbutton(_("Annotate today in day, week, and month views"),
- 		   PREF_DATEBOOK_HI_TODAY, vbox_datebook, cb_checkbox_set_pref);
  
-    /* Show number of years on aniversaries in month and week view */
-    add_checkbutton(
- 	_("Append years on anniversaries in day, week, and month views"),
- 	PREF_DATEBOOK_ANNI_YEARS, vbox_datebook,
- 	cb_checkbox_set_pref);
- 
-
    /**********************************************************************/
    /* Address preference tab */
 
+   /* Radio box to choose which database to use: Address/Contacts */
+   group = NULL;
+   radio_button_address_version[0] = 
+     gtk_radio_button_new_with_label(group, _("Use Address database (Palm OS < 5.2.1)"));
+   group = gtk_radio_button_group(GTK_RADIO_BUTTON(radio_button_address_version[0]));
+   radio_button_address_version[1] = 
+     gtk_radio_button_new_with_label(group, _("Use Contacts database (Palm OS > 5.2)"));
+   group = gtk_radio_button_group(GTK_RADIO_BUTTON(radio_button_address_version[1]));
+   gtk_box_pack_start(GTK_BOX(vbox_address), radio_button_address_version[0],
+		      FALSE, FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(vbox_address), radio_button_address_version[1],
+		      FALSE, FALSE, 0);
+
+   gtk_signal_connect(GTK_OBJECT(radio_button_address_version[0]), "pressed",
+		      GTK_SIGNAL_FUNC(cb_radio_set_pref),
+		      GINT_TO_POINTER((PREF_ADDRESS_VERSION<<16)|0));
+   gtk_signal_connect(GTK_OBJECT(radio_button_address_version[1]), "pressed",
+		      GTK_SIGNAL_FUNC(cb_radio_set_pref),
+		      GINT_TO_POINTER((PREF_ADDRESS_VERSION<<16)|1));
+
+   get_pref(PREF_ADDRESS_VERSION, &ivalue, NULL);
+   if (ivalue) {
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button_address_version[1]), TRUE);
+   } else {
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button_address_version[0]), TRUE);
+   }
+
+   /* Separate database selection from less important options */
+   hseparator = gtk_hseparator_new();
+   gtk_box_pack_start(GTK_BOX(vbox_address), hseparator, FALSE, FALSE, 3);
+
    /* Command to use for e-mailing from address book */
    hbox_temp = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox_address), hbox_temp, FALSE, FALSE, 0);
@@ -577,7 +749,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    label = gtk_label_new(_("Mail Command"));
    gtk_box_pack_start(GTK_BOX(hbox_temp), label, FALSE, FALSE, 0);
 
-   mail_command_entry = gtk_entry_new_with_max_length(MAX_PREF_VALUE - 2);
+   mail_command_entry = gtk_entry_new_with_max_length(MAX_PREF_LEN - 2);
 
    get_pref(PREF_MAIL_COMMAND, &ivalue, &cstr);
    if (cstr) {
@@ -595,15 +767,49 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    /**********************************************************************/
    /* ToDo preference tab */
 
-   /* The hide completed check box */
+   /* FIXME: undef when support for Task has been coded */
+#if 0
+   /* Radio box to choose which database to use: Todo/Task */
+   group = NULL;
+   radio_button_task_version[0] = 
+     gtk_radio_button_new_with_label(group, _("Use ToDo database (Palm OS < 5.2.1)"));
+   group = gtk_radio_button_group(GTK_RADIO_BUTTON(radio_button_todo_version[0]));
+   radio_button_todo_version[1] = 
+     gtk_radio_button_new_with_label(group, _("Use Task database (Palm OS > 5.2)"));
+   group = gtk_radio_button_group(GTK_RADIO_BUTTON(radio_button_todo_version[1]));
+   gtk_box_pack_start(GTK_BOX(vbox_todo), radio_button_todo_version[0],
+		      FALSE, FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(vbox_todo), radio_button_todo_version[1],
+		      FALSE, FALSE, 0);
+
+   gtk_signal_connect(GTK_OBJECT(radio_button_todo_version[0]), "pressed",
+		      GTK_SIGNAL_FUNC(cb_radio_set_pref),
+		      GINT_TO_POINTER((PREF_TODO_VERSION<<16)|0));
+   gtk_signal_connect(GTK_OBJECT(radio_button_todo_version[1]), "pressed",
+		      GTK_SIGNAL_FUNC(cb_radio_set_pref),
+		      GINT_TO_POINTER((PREF_TODO_VERSION<<16)|1));
+
+   get_pref(PREF_TODO_VERSION, &ivalue, NULL);
+   if (ivalue) {
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button_todo_version[1]), TRUE);
+   } else {
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button_todo_version[0]), TRUE);
+   }
+
+   /* Separate database selection from less important options */
+   hseparator = gtk_hseparator_new();
+   gtk_box_pack_start(GTK_BOX(vbox_address), hseparator, FALSE, FALSE, 3);
+#endif
+
+   /* hide completed check box */
    add_checkbutton(_("Hide Completed ToDos"),
 		   PREF_TODO_HIDE_COMPLETED, vbox_todo, cb_checkbox_set_pref);
 
-   /* The hide todos not yet due check box */
+   /* hide todos not yet due check box */
    add_checkbutton(_("Hide ToDos not yet due"),
 		   PREF_TODO_HIDE_NOT_DUE, vbox_todo, cb_checkbox_set_pref);
 
-   /* The record todo completion date check box */
+   /* record todo completion date check box */
    add_checkbutton(_("Record Completion Date"),
 		   PREF_TODO_COMPLETION_DATE, vbox_todo, cb_checkbox_set_pref);
 
@@ -620,7 +826,8 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    add_checkbutton(_("Use default number of days due"),
 		   PREF_TODO_DAYS_DUE, hbox_temp, cb_checkbox_set_pref);
 
-   todo_days_due_entry = gtk_entry_new_with_max_length(MAX_PREF_VALUE - 2);
+   todo_days_due_entry = gtk_entry_new_with_max_length(MAX_PREF_LEN - 2);
+   entry_set_multiline_truncate(GTK_ENTRY(todo_days_due_entry), TRUE);
    get_pref(PREF_TODO_DAYS_TILL_DUE, &ivalue, &cstr);
    temp[0]='\0';
    g_snprintf(temp, sizeof(temp), "%ld", ivalue);
@@ -635,35 +842,72 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
 
    /**********************************************************************/
    /* Memo preference tab */
+   /* Radio box to choose which database to use: Memo/Memos/Memo32 */
+   group = NULL;
+   radio_button_memo_version[0] = 
+     gtk_radio_button_new_with_label(group, _("Use Memo database (Palm OS < 5.2.1)"));
+   group = gtk_radio_button_group(GTK_RADIO_BUTTON(radio_button_memo_version[0]));
+   radio_button_memo_version[1] = 
+     gtk_radio_button_new_with_label(group, _("Use Memos database (Palm OS > 5.2)"));
+   group = gtk_radio_button_group(GTK_RADIO_BUTTON(radio_button_memo_version[1]));
+   radio_button_memo_version[2] = 
+     gtk_radio_button_new_with_label(group, _("Use Memo32 database (pedit32)"));
+   group = gtk_radio_button_group(GTK_RADIO_BUTTON(radio_button_memo_version[2]));
+   gtk_box_pack_start(GTK_BOX(vbox_memo), radio_button_memo_version[0],
+		      FALSE, FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(vbox_memo), radio_button_memo_version[1],
+		      FALSE, FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(vbox_memo), radio_button_memo_version[2],
+		      FALSE, FALSE, 0);
 
-   /* Memo32 check box */
-   add_checkbutton(_("Use Memo32 (pedit32)"),
-		   PREF_MEMO32_MODE, vbox_memo, cb_checkbox_set_pref);
+   gtk_signal_connect(GTK_OBJECT(radio_button_memo_version[0]), "pressed",
+		      GTK_SIGNAL_FUNC(cb_radio_set_pref),
+		      GINT_TO_POINTER((PREF_MEMO_VERSION<<16)|0));
+   gtk_signal_connect(GTK_OBJECT(radio_button_memo_version[1]), "pressed",
+		      GTK_SIGNAL_FUNC(cb_radio_set_pref),
+		      GINT_TO_POINTER((PREF_MEMO_VERSION<<16)|1));
+   gtk_signal_connect(GTK_OBJECT(radio_button_memo_version[2]), "pressed",
+		      GTK_SIGNAL_FUNC(cb_radio_set_pref),
+		      GINT_TO_POINTER((PREF_MEMO_VERSION<<16)|2));
+
+   get_pref(PREF_MEMO_VERSION, &ivalue, NULL);
+   switch (ivalue) {
+    case 0:
+    default:
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button_memo_version[0]), TRUE);
+      break;
+    case 1:
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button_memo_version[1]), TRUE);
+      break;
+    case 2:
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button_memo_version[2]), TRUE);
+      break;
+   }
 
    /**********************************************************************/
    /* Alarms preference tab */
 
-   /* Show open alarm windows check box */
+   /* Open alarm windows check box */
    add_checkbutton(_("Open alarm windows for appointment reminders"),
 		   PREF_OPEN_ALARM_WINDOWS, vbox_alarms, cb_checkbox_set_pref);
 
-   /* Show open alarm windows check box */
+   /* Execute alarm command check box */
    add_checkbutton(_("Execute this command"),
 		   PREF_DO_ALARM_COMMAND, vbox_alarms, cb_checkbox_set_pref);
 
-   /* Shell warning */
+   /* Shell warning label */
    label = gtk_label_new(_("WARNING: executing arbitrary shell commands can be dangerous!!!"));
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
    gtk_box_pack_start(GTK_BOX(vbox_alarms), label, FALSE, FALSE, 0);
 
-   /* Alarm Command */
+   /* Alarm Command to execute */
    hbox_temp = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox_alarms), hbox_temp, FALSE, FALSE, 0);
 
    label = gtk_label_new(_("Alarm Command"));
    gtk_box_pack_start(GTK_BOX(hbox_temp), label, FALSE, FALSE, 10);
 
-   alarm_command_entry = gtk_entry_new_with_max_length(MAX_PREF_VALUE - 2);
+   alarm_command_entry = gtk_entry_new_with_max_length(MAX_PREF_LEN - 2);
    get_pref(PREF_ALARM_COMMAND, &ivalue, &cstr);
    if (cstr) {
       gtk_entry_set_text(GTK_ENTRY(alarm_command_entry), cstr);
@@ -691,10 +935,12 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_box_pack_start(GTK_BOX(vbox_alarms), label, FALSE, FALSE, 0);
 #else
    label = gtk_label_new(_("%D (description substitution) is disabled in this build"));
+   gtk_widget_set_sensitive(label, FALSE);
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_box_pack_start(GTK_BOX(vbox_alarms), label, FALSE, FALSE, 0);
 
    label = gtk_label_new(_("%N (note substitution) is disabled in this build"));
+   gtk_widget_set_sensitive(label, FALSE);
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_box_pack_start(GTK_BOX(vbox_alarms), label, FALSE, FALSE, 0);
 #endif
@@ -702,28 +948,24 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    /**********************************************************************/
    /* Conduits preference tab */
 
-   /* Show sync datebook check box */
+   /* Sync datebook check box */
    add_checkbutton(_("Sync datebook"),
 		   PREF_SYNC_DATEBOOK, vbox_conduits, cb_checkbox_set_pref);
 
-   /* Show sync address check box */
+   /* Sync address check box */
    add_checkbutton(_("Sync address"),
 		   PREF_SYNC_ADDRESS, vbox_conduits, cb_checkbox_set_pref);
 
-   /* Show sync todo check box */
+   /* Sync todo check box */
    add_checkbutton(_("Sync todo"),
 		   PREF_SYNC_TODO, vbox_conduits, cb_checkbox_set_pref);
 
-   /* Show sync memo check box */
+   /* Sync memo check box */
    add_checkbutton(_("Sync memo"),
 		   PREF_SYNC_MEMO, vbox_conduits, cb_checkbox_set_pref);
 
-   /* Show sync Memo32 check box */
-   add_checkbutton(_("Sync memo32 (pedit32)"),
-		   PREF_SYNC_MEMO32, vbox_conduits, cb_checkbox_set_pref);
-
 #ifdef ENABLE_MANANA
-   /* Show sync Ma~nana check box */
+   /* Show sync Manana check box */
    add_checkbutton(_("Sync Manana"),
 		   PREF_SYNC_MANANA, vbox_conduits, cb_checkbox_set_pref);
 #endif
@@ -742,7 +984,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
       for (temp_list = plugin_list; temp_list; temp_list = temp_list->next) {
 	 Pplugin = (struct plugin_s *)temp_list->data;
 	 if (Pplugin) {
-	    /* Make a checkbox for each plugin */
+	    /* Make a Sync checkbox for each plugin */
 	    g_snprintf(temp, sizeof(temp), _("Sync %s (%s)"), Pplugin->name, Pplugin->full_path);
 	    checkbutton = gtk_check_button_new_with_label(temp);
 	    gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
@@ -759,11 +1001,16 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
 
 #endif
 
-   /* Create a "Done" button */
-   button = gtk_button_new_with_label(_("Done"));
+   /* Done button */
+   hbox_temp = gtk_hbutton_box_new();
+   gtk_button_box_set_layout(GTK_BUTTON_BOX (hbox_temp), GTK_BUTTONBOX_END);
+   gtk_box_pack_start(GTK_BOX(vbox), hbox_temp, FALSE, FALSE, 1);
+
+   button = gtk_button_new_from_stock(GTK_STOCK_OK);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_quit), window);
-   gtk_box_pack_end(GTK_BOX(vbox), button, FALSE, FALSE, 0);
+   gtk_box_pack_end(GTK_BOX(hbox_temp), button, FALSE, FALSE, 0);
 
    gtk_widget_show_all(window);
 }
+
