@@ -1,4 +1,4 @@
-/* $Id: print.c,v 1.29 2005/06/24 14:03:13 rousseau Exp $ */
+/* $Id: print.c,v 1.43 2009/05/06 20:14:00 rousseau Exp $ */
 
 /*******************************************************************************
  * print.c
@@ -20,26 +20,38 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ******************************************************************************/
 
+/********************************* Includes ***********************************/
 #include "config.h"
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
 #include <ctype.h>
-#include <stdlib.h>
+
+#include "print.h"
+#include "print_headers.h"
+#include "print_logo.h"
 #include "datebook.h"
 #include "address.h"
 #include "todo.h"
 #include "sync.h"
 #include "prefs.h"
-#include "print.h"
-#include "print_logo.h"
-#include "print_headers.h"
 #include "log.h"
 #include "i18n.h"
 #ifdef HAVE_LOCALE_H
-#include <locale.h>
+#  include <locale.h>
 #endif
 
+/********************************* Constants **********************************/
+#ifdef JPILOT_PRINTABLE
+#  define FLAG_CHAR 'A'
+#  define Q_FLAG_CHAR "A"
+#else
+#  define FLAG_CHAR 010
+#  define Q_FLAG_CHAR "\\010"
+#endif
+
+/******************************* Global vars **********************************/
 static FILE *out;
 static int first_hour, first_min, last_hour, last_min;
 extern int datebook_category;
@@ -56,15 +68,8 @@ char *PaperSizes[] = { "Letter", "Legal", "Statement", "Tabloid", "Ledger",
                        "C1", "C2", "C3", "C4", "C5", "C6", "C7", "DL",
                        "Filo" };
 
-#ifdef JPILOT_PRINTABLE
-#define FLAG_CHAR 'A'
-#define Q_FLAG_CHAR "A"
-#else
-#define FLAG_CHAR 010
-#define Q_FLAG_CHAR "\\010"
-#endif
-
-FILE *print_open()
+/****************************** Main Code *************************************/
+FILE *print_open(void)
 {
    const char *command;
 
@@ -81,14 +86,14 @@ void print_close(FILE *f)
    pclose(f);
 }
 
-int courier_12()
+int courier_12(void)
 {
    /* fprintf(out, "/Courier 12 selectfont\n"); */
    fprintf(out, "%cC12\n", FLAG_CHAR);
    return EXIT_SUCCESS;
 }
 
-int courier_bold_12()
+int courier_bold_12(void)
 {
    /* fprintf(out, "/Courier-Bold 12 selectfont\n"); */
    fprintf(out, "%cCB12\n", FLAG_CHAR);
@@ -108,7 +113,7 @@ int puttext(float x, float y, char *text)
    char *buf;
 
    len = strlen(text);
-   buf = (char *) malloc(2 * len + 1);
+   buf = malloc(2 * len + 1);
    memset(buf, 0, 2 * len + 1);
    ps_strncat(buf, text, 2 * len);
    fprintf(out, "%g inch %g inch moveto (%s) show\n", x, y, buf);
@@ -116,7 +121,7 @@ int puttext(float x, float y, char *text)
    return EXIT_SUCCESS;
 }
 
-int header()
+int header(void)
 {
    time_t ltime;
 
@@ -483,7 +488,7 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
    get_pref(PREF_FDOW, &fdow, NULL);
 
    fprintf(out,
-	   "(%s, %d) %d (%s) (%s version %s) %ld InitialisePage\n\n",
+	   "(%s %d) %d (%s) (%s version %s) %ld InitialisePage\n\n",
 	   MonthNames[date_in->tm_mon], date_in->tm_year + 1900,
 	   date.tm_wday,
 	   ctime(&ltime),
@@ -528,7 +533,7 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
 	    desc[0]='\0';
 	    if (temp_al->mappt.appt.description) {
 	       ps_strncat(desc, temp_al->mappt.appt.description, 100);
-	       desc[100]='\0';
+	       desc[sizeof(desc)-1]='\0';
 	    }
 	    remove_cr_lfs(desc);
 	    fprintf(out, "%s (%s) %simedItem\n", tmp, desc,
@@ -539,18 +544,15 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
 
    /*------------------------------------------------------------------*/
    memcpy(&date, date_in, sizeof(struct tm));
-   sub_months_from_date(&date, 1);
    date.tm_mday = 1;	/* Go to the first of the month */
    mktime(&date);
+   sub_months_from_date(&date, 1);
    strftime(desc, sizeof(desc), "(%B %Y) %w ", &date);
    fprintf(out, "\n\n%%----------------------------------------\n"
            "%% Now generate the small months\n\n"
            "%s %d ", desc, days_in_mon(&date));
 
-   memcpy(&date, date_in, sizeof(struct tm));
-   add_months_to_date(&date, 1);
-   date.tm_mday = 1;	/* Go to the first of the month */
-   mktime(&date);
+   add_months_to_date(&date, 2);
    strftime(desc, sizeof(desc), "(%B %Y) %w ", &date);
    fprintf(out, "%s %d SmallMonths\n", desc, days_in_mon(&date));
 
@@ -629,12 +631,14 @@ int print_weeks_appts(struct tm *date_in, PaperSize paper_size)
    long use_db3_tags;
    struct db4_struct db4;
 #endif
+#ifdef HAVE_LOCALE_H
+   char *current_locale;
+#endif
 
 #ifdef ENABLE_DATEBK
    get_pref(PREF_USE_DB3, &use_db3_tags, NULL);
 #endif
 #ifdef HAVE_LOCALE_H
-   char *current_locale;
    current_locale = setlocale(LC_NUMERIC,"C");
 #endif
 
@@ -783,13 +787,11 @@ int print_weeks_appts(struct tm *date_in, PaperSize paper_size)
      return EXIT_SUCCESS;
 }
 
-/*----------------------------------------------------------------------
- *
+/*
  * Address code
- *
- *----------------------------------------------------------------------*/
+ */
 
-int print_address_header()
+int print_address_header(void)
 {
    time_t ltime;
    struct tm *date;
@@ -947,7 +949,7 @@ int print_address_header()
 	   "pop pop pop ( )\n"
 	   "} if\n"
 	   "("Q_FLAG_CHAR"END) search { %%if\n"
-	   "   stop\n"
+	   "   showpage stop\n"
 	   "} if\n"
 	   "("Q_FLAG_CHAR"C12) search {\n"
 	   "/Courier-ISOLatin1 12 selectfont\n"
@@ -1008,20 +1010,26 @@ int print_address_header()
    return EXIT_SUCCESS;
 }
 
-int print_addresses(AddressList *address_list)
+
+int print_contacts(ContactList *contact_list, struct ContactAppInfo *contact_app_info,
+		   address_schema_entry *schema, int schema_size)
 {
    long one_rec_per_page;
    long lines_between_recs;
-   AddressList *temp_al;
-   struct AddressAppInfo address_app_info;
-   struct Address *addr;
+   ContactList *temp_cl;
+   MyContact *mcont;
    int show1, show2, show3;
-   int j, i, i2;
-   int len;
-   /*This is because the palm doesn\'t show the address entries in order */
-   int order[22]={0,1,13,2,3,4,5,6,7,8,9,10,11,12,14,15,16,17,18,19,20,21
-   };
+   int i;
+   int address_i, phone_i, IM_i;
    char str[100];
+   char spaces[24];
+   char birthday_str[255];
+   const char *pref_date;
+   char *utf;
+   long char_set;
+#ifdef HAVE_LOCALE_H
+   char *current_locale;
+#endif
 
    out = print_open();
    if (!out) {
@@ -1029,45 +1037,54 @@ int print_addresses(AddressList *address_list)
    }
 
 #ifdef HAVE_LOCALE_H
-   char *current_locale;
    current_locale = setlocale(LC_NUMERIC,"C");
 #endif
 
-   get_address_app_info(&address_app_info);
+   memset(spaces, ' ', sizeof(spaces));
+
+   get_pref(PREF_CHAR_SET, &char_set, NULL);
 
    print_address_header();
 
-   if (address_app_info.sortByCompany) {
-      show1=2; /*company */
-      show2=0; /*last name */
-      show3=1; /*first name */
-   } else {
-      show1=0; /*last name */
-      show2=1; /*first name */
-      show3=2; /*company */
+   switch (addr_sort_order) {
+    case SORT_BY_LNAME:
+    default:
+      show1=contLastname;
+      show2=contFirstname;
+      show3=contCompany;
+      break;
+    case SORT_BY_FNAME:
+      show1=contFirstname;
+      show2=contLastname;
+      show3=contCompany;
+      break;
+    case SORT_BY_COMPANY:
+      show1=contCompany;
+      show2=contLastname;
+      show3=contFirstname;
+      break;
    }
 
-#define NUM_ADDRESS_ENTRIES 19
    get_pref(PREF_PRINT_ONE_PER_PAGE, &one_rec_per_page, NULL);
    get_pref(PREF_NUM_BLANK_LINES, &lines_between_recs, NULL);
 
-   for (temp_al = address_list; temp_al; temp_al=temp_al->next) {
+   for (temp_cl = contact_list; temp_cl; temp_cl=temp_cl->next) {
 
       fprintf(out, "%cHLINE\n", FLAG_CHAR);
 
       str[0]='\0';
-      if (temp_al->maddr.addr.entry[show1] || temp_al->maddr.addr.entry[show2]) {
-	 if (temp_al->maddr.addr.entry[show1] && temp_al->maddr.addr.entry[show2]) {
-	    g_snprintf(str, sizeof(str), "%s, %s", temp_al->maddr.addr.entry[show1], temp_al->maddr.addr.entry[show2]);
+      if (temp_cl->mcont.cont.entry[show1] || temp_cl->mcont.cont.entry[show2]) {
+	 if (temp_cl->mcont.cont.entry[show1] && temp_cl->mcont.cont.entry[show2]) {
+	    g_snprintf(str, sizeof(str), "%s, %s", temp_cl->mcont.cont.entry[show1], temp_cl->mcont.cont.entry[show2]);
 	 }
-	 if (temp_al->maddr.addr.entry[show1] && ! temp_al->maddr.addr.entry[show2]) {
-	    strncpy(str, temp_al->maddr.addr.entry[show1], 48);
+	 if (temp_cl->mcont.cont.entry[show1] && ! temp_cl->mcont.cont.entry[show2]) {
+	    strncpy(str, temp_cl->mcont.cont.entry[show1], 48);
 	 }
-	 if (! temp_al->maddr.addr.entry[show1] && temp_al->maddr.addr.entry[show2]) {
-	    strncpy(str, temp_al->maddr.addr.entry[show2], 48);
+	 if (! temp_cl->mcont.cont.entry[show1] && temp_cl->mcont.cont.entry[show2]) {
+	    strncpy(str, temp_cl->mcont.cont.entry[show2], 48);
 	 }
-      } else if (temp_al->maddr.addr.entry[show3]) {
-	    strncpy(str, temp_al->maddr.addr.entry[show3], 48);
+      } else if (temp_cl->mcont.cont.entry[show3]) {
+	    strncpy(str, temp_cl->mcont.cont.entry[show3], 48);
       } else {
 	    strcpy(str, "-Unnamed-");
       }
@@ -1076,24 +1093,64 @@ int print_addresses(AddressList *address_list)
       fprintf(out, "%s\n", str);
       courier_12();
 
-      addr = &(temp_al->maddr.addr);
-      for (i=0; i<NUM_ADDRESS_ENTRIES; i++) {
-	 i2=order[i];
-	 if (addr->entry[i2]) {
-	    if (i2>2 && i2<8) {
-	       fprintf(out, "%s: ", address_app_info.phoneLabels[addr->phoneLabel[i2-3]]);
-	       len = strlen(address_app_info.phoneLabels[addr->phoneLabel[i2-3]]);
-	    } else {
-	       fprintf(out, "%s: ", address_app_info.labels[i2]);
-	       len = strlen(address_app_info.labels[i2]);
+      mcont = &(temp_cl->mcont);
+      address_i=phone_i=IM_i=0;
+      for (i=0; i<schema_size; i++) {
+	 /* Get the entry texts */
+	 if (mcont->cont.entry[schema[i].record_field]) {
+	    switch (schema[i].type) {
+	     case ADDRESS_GUI_IM_MENU_TEXT:
+	       g_snprintf(str, 18, "%s:%s", contact_app_info->IMLabels[mcont->cont.IMLabel[IM_i]], spaces);
+	       fprintf(out, "%s", str);
+	       IM_i++;
+	       break;
+	     case ADDRESS_GUI_DIAL_SHOW_PHONE_MENU_TEXT:
+	       g_snprintf(str, 18, "%s:%s", contact_app_info->phoneLabels[mcont->cont.phoneLabel[phone_i]], spaces);
+	       fprintf(out, "%s", str);
+	       phone_i++;
+	       break;
+	     case ADDRESS_GUI_ADDR_MENU_TEXT:
+	       g_snprintf(str, 18, "%s:%s", contact_app_info->addrLabels[mcont->cont.addressLabel[address_i]], spaces);
+	       fprintf(out, "%s", str);
+	       address_i++;
+	       break;
+	     default:
+	       if (contact_app_info->labels[schema[i].record_field]) {
+		  utf = charset_p2newj(contact_app_info->labels[schema[i].record_field], 16, char_set);
+		  g_snprintf(str, 18, "%s:%s", utf, spaces);
+		  fprintf(out, "%s", str);
+		  g_free(utf);
+	       }
+	       else {
+		  g_snprintf(str, 18, ":%s", spaces);
+		  fprintf(out, "%s", str);
+	       }
 	    }
-	    for (j=16-len; j; j--) {
-	       fprintf(out, " ");
+	    switch (schema[i].type) {
+	     case ADDRESS_GUI_LABEL_TEXT:
+	     case ADDRESS_GUI_DIAL_SHOW_PHONE_MENU_TEXT:
+	     case ADDRESS_GUI_IM_MENU_TEXT:
+	     case ADDRESS_GUI_ADDR_MENU_TEXT:
+	     case ADDRESS_GUI_WEBSITE_TEXT:
+	       f_indent_print(out, 17, mcont->cont.entry[schema[i].record_field]);
+	       fprintf(out, "\n");
+	       break;
+	     case ADDRESS_GUI_BIRTHDAY:
+	       if (mcont->cont.birthdayFlag) {
+		  birthday_str[0]='\0';
+		  get_pref(PREF_SHORTDATE, NULL, &pref_date);
+		  strftime(birthday_str, sizeof(birthday_str), pref_date, &(mcont->cont.birthday));
+		  g_snprintf(str, 18, "%s:%s", contact_app_info->labels[schema[i].record_field] ? contact_app_info->labels[schema[i].record_field] : "",
+			     spaces);
+		  fprintf(out, "%s", str);
+		  f_indent_print(out, 17, birthday_str);
+		  fprintf(out, "\n");
+	       }
+	       break;
 	    }
-	    f_indent_print(out, 18, addr->entry[i2]);
-	    fprintf(out, "\n");
 	 }
       }
+
       if (one_rec_per_page) {
 	 fprintf(out, "%cLINEFEED\n", FLAG_CHAR);
       } else {
@@ -1113,9 +1170,7 @@ int print_addresses(AddressList *address_list)
 }
 
 /*
- *
  * ToDo code
- *
  */
 int print_todos(ToDoList *todo_list, char *category_name)
 {
@@ -1128,6 +1183,9 @@ int print_todos(ToDoList *todo_list, char *category_name)
    char str[100];
    time_t ltime;
    struct tm *now;
+#ifdef HAVE_LOCALE_H
+   char *current_locale;
+#endif
 
    out = print_open();
    if (!out) {
@@ -1135,7 +1193,6 @@ int print_todos(ToDoList *todo_list, char *category_name)
    }
 
 #ifdef HAVE_LOCALE_H
-   char *current_locale;
    current_locale = setlocale(LC_NUMERIC,"C");
 #endif
 
@@ -1174,7 +1231,7 @@ int print_todos(ToDoList *todo_list, char *category_name)
 	 char *buf;
 
 	 len = strlen(todo->description);
-	 buf = (char *) malloc(2 * len + 1);
+	 buf = malloc(2 * len + 1);
 	 memset(buf, 0, 2 * len + 1);
 	 ps_strncat(buf, todo->description, 2 * len);
 
@@ -1189,7 +1246,7 @@ int print_todos(ToDoList *todo_list, char *category_name)
 	 char *buf;
 
 	 len = strlen(todo->note);
-	 buf = (char *) malloc(2 * len + 1);
+	 buf = malloc(2 * len + 1);
 	 memset(buf, 0, 2 * len + 1);
 	 ps_strncat(buf, todo->note, 2 * len);
 
@@ -1215,9 +1272,7 @@ int print_todos(ToDoList *todo_list, char *category_name)
    return EXIT_SUCCESS;
 }
 /*
- *
  * Memo code
- *
  */
 int print_memos(MemoList *memo_list)
 {
@@ -1226,6 +1281,9 @@ int print_memos(MemoList *memo_list)
    MemoList *temp_l;
    struct Memo *memo;
    int i;
+#ifdef HAVE_LOCALE_H
+   char *current_locale;
+#endif
 
    out = print_open();
    if (!out) {
@@ -1233,7 +1291,6 @@ int print_memos(MemoList *memo_list)
    }
 
 #ifdef HAVE_LOCALE_H
-   char *current_locale;
    current_locale = setlocale(LC_NUMERIC,"C");
 #endif
 
@@ -1272,3 +1329,4 @@ int print_memos(MemoList *memo_list)
 
    return EXIT_SUCCESS;
 }
+
