@@ -12,7 +12,8 @@ import jppy
 import os
 
 class pane:
-    def __init__(self, gladefile, wTree):
+    def __init__(self, env, gladefile, wTree):
+        self.env   = env
         self.wTree = wTree
         self.gladefile = gladefile
         self.iter = None
@@ -21,7 +22,7 @@ class pane:
             raise NotImplementedError("Model not provided")
         if not self.view:
             raise NotImplementedError("View not provided")            
-        if not self.recordCategoryOpenMenu:
+        if not self.recordCategoryComboBox:
             raise NotImplementedError("Record category widget not provided")
         if not self.categoryFilterVBox:
             raise NotImplementedError("Category filter vbox not provided")
@@ -54,12 +55,13 @@ class pane:
         # category filter selector
         # as to be build at init time, because
         # the categories can change
-        menu = gtk.Menu()
+        liststore = gtk.ListStore(gobject.TYPE_STRING)
+        self.recordCategoryComboBox.set_model(liststore)
+        cell = gtk.CellRendererText()
+        self.recordCategoryComboBox.pack_start(cell, True)
+        self.recordCategoryComboBox.add_attribute(cell, 'text', 0)
         for category in self.model.available_categories():
-            menuitem = gtk.MenuItem(label=category)
-            menuitem.show()
-            menu.append(menuitem)
-        self.recordCategoryOpenMenu.set_menu(menu)
+            liststore.append([category])
 
         self.categoryCheckboxes = []
         self.categoryFilterVBox.hide()
@@ -91,13 +93,15 @@ class pane:
                     buffer.connect_object("changed", self.on_textview_change, w)
                 elif isinstance(w,gtk.OptionMenu):
                     w.connect("changed", self.on_optionmenu_change)
+                elif isinstance(w,gtk.ComboBox):
+                    w.connect("changed", self.on_combobox_change)
                 elif isinstance(w,gtk.CheckButton):
                     w.connect("toggled", self.on_checkbutton_change)
                 elif isinstance(w,gnome.ui.DateEdit):
                     w.connect("date-changed", self.on_dateedit_change)
                     w.connect("time-changed", self.on_dateedit_change)
                 else:
-                    print "How do I deal with type %s for %s?" % (type(w), attribute)
+                    self.env.log.warn("How do I deal with type %s for %s?" % (type(w), attribute))
 
 
         # initial display
@@ -135,16 +139,21 @@ class pane:
         model.save_to_disk()
 
     def on_RevertButton(self, widget, model):
-        path = self.model.get_path(self.iter)
-        model.rescan_database(save=0)
-        try:
-            newiter = self.model.get_iter(path)
-        except ValueError:
-            path = (len(self.model.records)-1,0)
-            newiter = self.model.get_iter(path)
-        selection = self.view.get_selection()
-        selection.select_iter(newiter)
-        self.view.scroll_to_cell(path)
+        record = self.model.get_value(self.iter,0)
+        if self.revert_record_confirmation(str(record),
+                                           record.unique_id == 0) == gtk.RESPONSE_OK:
+            path = self.model.get_path(self.iter)
+            # we can probably find a way to avoid this?
+            model.rescan_database(save=0)
+            try:
+                newiter = self.model.get_iter(path)
+            except ValueError:
+                path = (len(self.model.records)-1,)
+                newiter = self.model.get_iter(path)
+            selection = self.view.get_selection()
+            selection.select_iter(newiter)
+            self.view.scroll_to_cell(path)
+            self.updateDetailsView(newiter)
 
     def on_DeleteButton(self, widget):
         record_name = str(self.model.get_value(self.iter,0))
@@ -152,6 +161,9 @@ class pane:
             path = self.model.get_path(self.iter)
             self.model.remove(self.iter)
 
+            if len(self.model.records) == 0:
+                return
+
             try:
                 newiter = self.model.get_iter(path)
             except ValueError:
@@ -176,8 +188,10 @@ class pane:
     def on_FilterButton(self, widget, focuswidgetname):
         self.wTree.get_widget(focuswidgetname).grab_focus()
 
-    def on_FilterTypeCheckbox(self, widget, entrywidgetname):
-        if widget.get_active() == True:
+    def on_FilterTypeCombo(self, widget, entrywidgetname):
+        if widget.get_active_text() == "Full Text":
+            self.wTree.get_widget(entrywidgetname).set_data('searchType', 'full')
+        elif widget.get_active_text() == "Names":
             self.wTree.get_widget(entrywidgetname).set_data('searchType', 'simple')
         else:
             self.wTree.get_widget(entrywidgetname).set_data('searchType', 'query')
@@ -222,6 +236,14 @@ class pane:
             num = widget.get_history()
             model.set_value(iter, n, num)
 
+    def on_combobox_change(self, widget):
+        iter = widget.get_data("iter")
+        if iter:
+            n = widget.get_data("attribute")
+            model = widget.get_data("model")
+            num = widget.get_active()
+            model.set_value(iter, n, num)
+
     def on_checkbutton_change(self, widget):
         iter = widget.get_data("iter")
         if iter:
@@ -247,7 +269,7 @@ class pane:
             model.set_value(iter, n, date)
 
     def on_columnentry_change(self, widget, row, value=None):
-        print widget, row, value
+        self.env.log.debug("%s %s %s" % (widget, row, value))
         if value == None:
             # we're toggling
             value = not widget.get_active()
@@ -304,6 +326,8 @@ class pane:
                         buffer.set_text(value)        
                     elif isinstance(widget,gtk.OptionMenu):
                         widget.set_history(int(value))
+                    elif isinstance(widget,gtk.ComboBox):
+                        widget.set_active(int(value))
                     elif isinstance(widget,gtk.CheckButton):
                         if value == 1:
                             widget.set_active(True)
@@ -314,11 +338,11 @@ class pane:
                             t = int(value.ticks())
                             widget.set_time(t)
                         except mx.DateTime.Error, e:
-                            print "WARNING: Date %s - %s" % (value, e)
-                            print "attribute: %s label: %s" % (attribute, label)
+                            self.env.log.warn("WARNING: Date %s - %s" % (value, e))
+                            self.env.log.warn("attribute: %s label: %s" % (attribute, label))
                     else:
-                        print "How do I deal with type %s for %s" % (type(widget),
-                                                                     attribute)
+                        self.env.log.warn("How do I deal with type %s for %s?" % (type(widget),
+                                                                                  attribute))
                     widget.set_data("iter", iter)
 
     def on_TreeView_selection_changed(self, selection):
@@ -342,6 +366,24 @@ class pane:
         dialog.destroy()
         return res
 
+    def revert_record_confirmation(self, name, new):
+        if new:
+            dialog = gtk.MessageDialog(None,                           # 
+                                       gtk.DIALOG_DESTROY_WITH_PARENT, # what parent ? ;-)
+                                       gtk.MESSAGE_QUESTION,
+                                       gtk.BUTTONS_OK_CANCEL,
+                                       "Throw away new record?")
+        else:
+            dialog = gtk.MessageDialog(None,                           # 
+                                       gtk.DIALOG_DESTROY_WITH_PARENT, # what parent ? ;-)
+                                       gtk.MESSAGE_QUESTION,
+                                       gtk.BUTTONS_OK_CANCEL,
+                                       "Revert record\n%s?" % name)
+        dialog.set_property('window_position',gtk.WIN_POS_MOUSE)
+        res = dialog.run()
+        dialog.destroy()
+        return res
+
 
     def on_Categories_toggled(self, widget, model, checkboxes):
         selected = []
@@ -358,7 +400,7 @@ class pane:
             return
 
         if widget.get_data("searchType") is None:
-            widget.set_data("searchType","simple")
+            widget.set_data("searchType","full")
 
         text = widget.get_text()
         if len(text) == 0 or len(text) > 2 or event.keyval == gtk.keysyms.Return:
@@ -371,10 +413,18 @@ class pane:
                     widget.modify_base(gtk.STATE_NORMAL,
                                        gtk.gdk.color_parse('#F00'))
                     return # don't bother running query
-            else:
+            elif widget.get_data("searchType") == "full":
                 model.setSearchTerm(text)
                 widget.modify_base(gtk.STATE_NORMAL,
                                    gtk.gdk.color_parse('#FFF'))
+            elif widget.get_data("searchType") == "simple":
+                model.setNamesFilterTerm(text)
+                widget.modify_base(gtk.STATE_NORMAL,
+                                   gtk.gdk.color_parse('#FFF'))
+            else:
+                widget.modify_base(gtk.STATE_NORMAL,
+                                   gtk.gdk.color_parse('#F00'))
+                return
             model.rescan_database()
             self.clearDetailsView()
             
@@ -436,7 +486,7 @@ class pane:
                 column.set_resizable(True)
                 column.set_reorderable(True)
                 if max_width:
-                    print "Fixing width at %d for %s" % (max_width, field_detail)
+                    self.env.log.debug("Fixing width at %d for %s" % (max_width, field_detail))
                     column.set_max_width(max_width)
                     column.set_min_width(200)                    
                 #column.set_max_width(400)
@@ -456,7 +506,7 @@ class pane:
         raise NotImplementedError
 
     def clearDetailsView(self):
-        print "Clearing"
+        self.env.log.debug("Clearing")
         self.iter = None        
         for n, attribute, label, widgetname, gtype, pytype in self.model.columns:
             if widgetname:
@@ -470,13 +520,15 @@ class pane:
                         buffer.set_text("")
                     elif isinstance(widget,gtk.OptionMenu):
                         widget.set_history(0)
+                    elif isinstance(widget,gtk.ComboBox):
+                        widget.set_active(-1)
                     elif isinstance(widget,gtk.CheckButton):
                         widget.checked = 0
                     elif isinstance(widget,gnome.ui.DateEdit):
                         widget.set_time(946684800) # 2000
                     else:
-                        print "How do I deal with type %s for %s?" % (type(widget),
-                                                                      attribute)
+                        self.env.log.warn("How do I deal with type %s for %s?" % (type(widget),
+                                                                                  attribute))
         selection = self.view.get_selection()
         try:
             iter = self.model.get_iter_first()
@@ -513,6 +565,6 @@ class pane:
             d.run()
             d.destroy()
             return
-        print cmd
+        self.env.log.info(cmd)
         os.system(cmd)
         
