@@ -37,22 +37,25 @@ from jppy.latex import letter
 
 
 class pane(base.pane):
-    def __init__(self, glade_path=os.path.join("@@python_module_prefix@@","gui")):
+    def __init__(self, env, glade_path):
 
         gladefile  = os.path.join(glade_path, "contacts.glade")
         wTree      = gtk.glade.XML(gladefile,"vpanedContacts")
         wTreeMenu  = gtk.glade.XML(gladefile,"contactsLogMenu")
         self.name  = "Contacts"
-        self.model = ContactsModel()
+        self.model = ContactsModel(env)
         self.main_vpaned = wTree.get_widget("vpanedContacts")
         self.view = wTree.get_widget("ContactsTreeView")
-        self.recordCategoryOpenMenu = wTree.get_widget("ContactsCategoryOptionMenu")
+        self.recordCategoryComboBox = wTree.get_widget("CategoryComboBox")
         self.categoryFilterVBox = wTree.get_widget("ContactsCategoryVBox")
 
         wTree.get_widget("contactsDialButton").set_sensitive(AsteriskManager != None)
 
+        wTree.get_widget("ContactsFilterWanted").set_active(0)
+        wTree.get_widget("ContactsFilterEntry").set_data('searchType', 'simple')
+
         try:
-            mode = jppy.config.get("sms","mode")
+            mode = env.config.get("sms","mode")
         except (ConfigParser.NoSectionError, ConfigParser.NoOptionError), e:
             mode = None
         if mode == "clickatell" and SMSManager == None:
@@ -69,7 +72,7 @@ class pane(base.pane):
         # unsupported fields
         wTree.get_widget("ContactsPrivateCheckButton").set_sensitive(False)
 
-        base.pane.__init__(self, gladefile, wTree)
+        base.pane.__init__(self, env, gladefile, wTree)
 
         self.translateGUI(wTree)
 
@@ -83,7 +86,7 @@ class pane(base.pane):
             "on_ContactsTreeView_button_press_event": (self.on_TreeView_button_press_event,
                                                        self.rightClickColumnMenu),
             "on_ContactsFilterButton_clicked": (self.on_FilterButton, "ContactsFilterEntry"),
-            "on_ContactsFilterCheckbox_toggled": (self.on_FilterTypeCheckbox, "ContactsFilterEntry"),
+            "on_ContactsFilterWanted_changed": (self.on_FilterTypeCombo, "ContactsFilterEntry"),
             "on_ContactsFilterEntry_keyrelease": (self.on_FilterEntry_keyrelease, self.model),
             
             "on_contactsRevertButton_clicked": (self.on_RevertButton, self.model),
@@ -181,18 +184,15 @@ class pane(base.pane):
             self.wTree.get_widget("contactsEmailButton").set_sensitive(contact['email'] != None)
             showphone = self.wTree.get_widget("Showphone")
             showphone.set_data("iter", None)
-            setting = showphone.get_history()
+            setting = showphone.get_active()
             menu = gtk.Menu()
             labels = self.model.get_value(self.iter,
                                           self.model.getColumnForNamesList(
                 self.model.findAttributeNumberByName("showphone")))
+            showphone.get_model().clear()
             for label in labels:
-                menuitem = gtk.MenuItem(label=label)
-                menuitem.show()
-                menu.append(menuitem)
-            showphone.remove_menu()
-            showphone.set_menu(menu)
-            showphone.set_history(setting)
+                showphone.append_text(label)
+            showphone.set_active(setting)
             showphone.set_data("iter", self.iter)
         
     def updateDetailsView(self, iter):
@@ -242,20 +242,20 @@ class pane(base.pane):
 
     def on_contactsPrintButton(self, widget):
         contact = self.model.get_value(self.iter,0)
-        render = contact_printer(jppy.addressBook(),contact)
+        render = contact_printer(self.env.addressBook,contact)
         render.display()
         self.model.set_value(self.iter,0,contact)
         self.updateDetailsView(self.iter)
 
     def on_contactsDialButton(self, widget):
         try:
-            host = jppy.config.get("asterisk","host")
-            port = jppy.config.get("asterisk","port")
-            user = jppy.config.get("asterisk","user")
-            password = jppy.config.get("asterisk","password")
-            context = jppy.config.get("asterisk","context")
-            phone = jppy.config.get("asterisk","phone")
-            dialprefix = jppy.config.get("asterisk","dialprefix")            
+            host = self.env.config.get("asterisk","host")
+            port = self.env.config.get("asterisk","port")
+            user = self.env.config.get("asterisk","user")
+            password = self.env.config.get("asterisk","password")
+            context = self.env.config.get("asterisk","context")
+            phone = self.env.config.get("asterisk","phone")
+            dialprefix = self.env.config.get("asterisk","dialprefix")            
         except ConfigParser.NoOptionError, e:
             d = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE,
                                   "Error with ~/.jppyrc: %s" % (e))
@@ -271,7 +271,7 @@ class pane(base.pane):
         dialog.get_widget("labelContext").set_text(context)
         dialog.get_widget("labelLocalExt").set_text(phone)
 
-        labels = jppy.addressBook().getPhoneLabels()
+        labels = self.env.addressBook.getPhoneLabels()
         for i, (value, label) in zip(range(1,8),
                                      self.model.get_value(self.iter,0)['phones_with_labels']):
             radio = dialog.get_widget("radiobutton%d" % i)
@@ -291,7 +291,7 @@ class pane(base.pane):
         elif response == gtk.RESPONSE_OK:
             for radio in dialog.get_widget("radiobutton1").get_group():
                 if radio.get_active() and radio.get_property('visible'):
-                    print "Dialing %s" % radio.get_data("number")
+                    self.env.log.info("Dialing %s" % radio.get_data("number"))
                     try:
                         m = AsteriskManager((host,int(port)), user, password, listen_events=False)
                         m.Originate(phone,
@@ -339,7 +339,7 @@ class pane(base.pane):
 
     def fetchSMSConfig(self):
         try:
-            mode = jppy.config.get("sms","mode")
+            mode = self.env.config.get("sms","mode")
         except (ConfigParser.NoSectionError, ConfigParser.NoOptionError), e:
             d = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE,
                                   "Error with ~/.jppyrc: No [sms]/mode set")
@@ -348,10 +348,10 @@ class pane(base.pane):
             return False            
         if mode == "clickatell":
             try:
-                smsconf = {'apiid': jppy.config.get("sms","apiid"),
-                           'user': jppy.config.get("sms","username"),
-                           'password': jppy.config.get("sms","password"),
-                           'frm': jppy.config.get("sms","from")}
+                smsconf = {'apiid': self.env.config.get("sms","apiid"),
+                           'user': self.env.config.get("sms","username"),
+                           'password': self.env.config.get("sms","password"),
+                           'frm': self.env.config.get("sms","from")}
             except ConfigParser.NoOptionError, e:
                 d = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE,
                                       "Error with ~/.jppyrc: %s" % (e))
@@ -359,12 +359,12 @@ class pane(base.pane):
                 d.destroy()
                 return False
             try:
-                smsconf['callback'] = jppy.config.get("sms","callback")
+                smsconf['callback'] = self.env.config.get("sms","callback")
             except ConfigParser.NoOptionError, e:
                 smsconf['callback'] = '0'
         else:
             try:
-                smsconf = {'command': jppy.config.get("sms","command")}
+                smsconf = {'command': self.env.config.get("sms","command")}
             except ConfigParser.NoOptionError, e:
                 d = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE,
                                       "Error with ~/.jppyrc: %s" % (e))
@@ -391,7 +391,7 @@ class pane(base.pane):
             dialog.get_widget("frameClickatell").hide()
             dialog.get_widget("labelShellCommand").set_text(smsconfig['command'])            
 
-        labels = jppy.addressBook().getPhoneLabels()
+        labels = self.env.addressBook.getPhoneLabels()
         for i, (value, label) in zip(range(1,8),
                                      self.model.get_value(self.iter,0)['phones_with_labels']):
             radio = dialog.get_widget("radiobuttonsms%d" % i)
@@ -423,7 +423,7 @@ class pane(base.pane):
             for radio in dialog.get_widget("radiobuttonsms1").get_group():
                 if radio.get_active() and radio.get_property('visible'):
                     targetnumber = radio.get_data("number")
-                    print "SMS to %s" % targetnumber
+                    self.env.log.info("SMS to %s" % targetnumber)
                     buffer = dialog.get_widget("textviewSMS").get_buffer()
                     text   = buffer.get_text(buffer.get_start_iter(),
                                              buffer.get_end_iter(),True)
@@ -490,7 +490,7 @@ class pane(base.pane):
         if response in (gtk.RESPONSE_CLOSE, gtk.RESPONSE_CANCEL):
             w.destroy()
         elif response == gtk.RESPONSE_OK:
-            print "File ", w.get_filename()
+            self.env.log.info("File %s", w.get_filename())
             try:
                 pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(w.get_filename(), 144, 144)
             except gobject.GError, e:
@@ -512,7 +512,7 @@ class pane(base.pane):
             pixbuf.save_to_callback(store_as_picture, "jpeg", user_data=contact)
             self.updateDetailsView(self.iter)
         else:
-            print "Unknown dialog response?"
+            self.env.log.warn("Unknown dialog response?")
             w.destroy()
 
     def on_contactsVcardButton(self, widget):
@@ -600,7 +600,7 @@ class pane(base.pane):
             if response in (gtk.RESPONSE_CLOSE, gtk.RESPONSE_CANCEL):
                 w.destroy()
             elif response == gtk.RESPONSE_OK:
-                print "File ", w.get_filename()
+                self.env.log.info("File ", w.get_filename())
                 try:
                     open(w.get_filename(),"w").write(text)
                 except Exception, e:
@@ -633,7 +633,7 @@ class pane(base.pane):
             filename = tempfile.mktemp(".vcf")
             open(filename,'w').write(text)
             for address in addresses:
-                print "Sending to %s" % address
+                self.env.log.info("Sending to %s" % address)
                 lightblue.obex.sendfile(address,11,filename)
             os.unlink(filename)
             dialog.destroy()
@@ -688,14 +688,16 @@ class pane(base.pane):
                     contact = self.model.get_value(iter,0)
 
                     if type == "label":
-                        l = contact_label(jppy.addressBook(), contact,
-                                          select=radio.get_data("address"),
-                                          sender=sender)
+                        l = contact_label(self.env)
+                        l.render(self.env.addressBook,
+                                 contact,
+                                 select=radio.get_data("address"),
+                                 sender=sender)
                         l.hardcopy(log=text)
                     elif type == "letter":
-                        l = letter(contact,
-                                   select=radio.get_data("address"),
-                                   sender=sender)
+                        l = letter(self.env)
+                        l.merge(contact,
+                                select=radio.get_data("address"))
                         l.display(log=text)                        
                     self.model.set_value(iter,0,contact)
                     self.updateDetailsView(iter)        
@@ -710,7 +712,7 @@ class pane(base.pane):
         w.connect_object("response", self.on_contactsAddLogResponse, w, buffer, self.iter)
 
     def on_contactsAddLogResponse(self, dialog, response, buffer, iter):
-        print buffer, iter
+        self.env.log.debug("%s %s" % (buffer, iter))
         if response == gtk.RESPONSE_OK:
             begin = buffer.get_start_iter()
             end   = buffer.get_end_iter()
