@@ -12,26 +12,6 @@ import ConfigParser
 
 import queryLanguage
 
-try:
-    use_pyuca = jppy.config.get('core','unicode-collation').upper() == "YES"
-except ConfigParser.NoOptionError, e:
-    use_pyuca = False
-    print "No unicode collator (disabled via configuration)"
-collator = None
-if use_pyuca:
-    try:
-        from pyuca import Collator
-        print "Loading collation keys..."
-        keysfilename = os.path.join("@@python_module_prefix@@",
-                                    "gui","allkeys.txt")
-        print "Attempting to load keys from %s" % keysfilename
-        collator = Collator(keysfilename)
-        print "loaded."
-    except Exception,e :
-        print "Failed to load collator, falling back to plain sort()"
-        print e
-
-
 pystring = types.StringType
 gstring  = gobject.TYPE_STRING
 
@@ -50,17 +30,24 @@ gpyobject = gobject.TYPE_PYOBJECT
 
 pylist = types.ListType
 
+collator = None
+
 class jppyModel(gtk.GenericTreeModel):
     filter = {}
 
-    def __init__(self):
+    def __init__(self, env):
+        global collator
+
 	gtk.GenericTreeModel.__init__(self)
 
         self.records = None
         self.search  = ""
         self.queryfilter = None
+        self.searchnames = []        
         self.categories  = []
 
+        self.env = env
+
         self.columns = []
 
         n = 1
@@ -76,6 +63,23 @@ class jppyModel(gtk.GenericTreeModel):
             self._labels = {}
             self.labels = []
 
+        try:
+            use_pyuca = self.env.config.get('core','unicode-collation').upper() == "YES"
+        except ConfigParser.NoOptionError, e:
+            use_pyuca = False
+            self.env.log.info("No unicode collator (disabled via configuration)")
+
+        if use_pyuca and collator is None:
+            try:
+                from pyuca import Collator
+                self.env.log.debug("Loading collation keys...")
+                keysfilename = os.path.join("@@prefix@@","share","jppy","allkeys.txt")
+                self.env.log.info("Attempting to load keys from %s" % keysfilename)
+                collator = Collator(keysfilename)
+                self.env.log.debug("loaded.")
+            except Exception,e :
+                self.env.log.warn("Failed to load collator: %s, falling back to plain sort()" % e)
+
     def collate(self, records):
         if collator:
             def func(record):
@@ -113,11 +117,18 @@ class jppyModel(gtk.GenericTreeModel):
             
     def setSearchTerm(self,str):
         self.search = str
-        self.queryfilter = None        
+        self.queryfilter = None
+        self.searchnames = []
 
     def setQueryFilterTerm(self,str):
         self.queryfilter = queryLanguage.parseQuery(str, self.querygrammer)
         self.search = ""
+        self.searchnames = []
+
+    def setNamesFilterTerm(self,str):
+        self.queryfilter = None
+        self.search = ""
+        self.searchnames = [x.lower() for x in str.split()]
 
     def setCategoryFilter(self,cats):
         self.categories = cats
@@ -132,11 +143,11 @@ class jppyModel(gtk.GenericTreeModel):
         if self.records:
             for contact in self.records:
                 if contact.unsaved_changes:
-                    print "Saved changes to %s" % contact
+                    self.env.log.info("Saved changes to %s" % contact)
                     self._database.save(contact)
 
     def new_record(self):
-        return self.newrecord()
+        return self._database.new()
 
     def get_new_records(self):
         self.records = self._database.records(search=self.search,
@@ -144,8 +155,19 @@ class jppyModel(gtk.GenericTreeModel):
         if self.queryfilter:
             self.records = queryLanguage.runQuery(self.queryfilter, self.records)
             if not isinstance(self.records, types.ListType):
-                print "WARNING: %s" % self.records
+                self.env.log.info("WARNING: %s" % self.records)
                 self.records = []
+        elif self.searchnames:
+            matched_records = []
+            for record in self.records:
+                include = True
+                for term in self.searchnames:
+                    # only supports Contacts for now....
+                    if not (term in record['firstname'].lower() or term in record['lastname'].lower()):
+                        include = False
+                if include:
+                    matched_records.append(record)
+            self.records = matched_records
         self.records = self.collate(self.records)
         
     def available_categories(self):
@@ -162,7 +184,7 @@ class jppyModel(gtk.GenericTreeModel):
         return filter(lambda r: r[0], cats)
     
     def rescan_database(self,save=1):
-        print "Scanning database"
+        self.env.log.debug("Scanning database")
         if save:
             self.save_to_disk()
             
@@ -170,7 +192,7 @@ class jppyModel(gtk.GenericTreeModel):
             before = len(self.records)
         else:
             before = 0
-        print self.search, self.categories
+        self.env.log.debug("%s %s" % (self.search, self.categories))
 
         self.get_new_records()
 
@@ -239,7 +261,7 @@ class jppyModel(gtk.GenericTreeModel):
     def remove(self, iter):
         path = self.get_path(iter)
         record = self.records[path[0]]
-        print "DELETING %s:%s" % (path, record)
+        self.env.log.info("DELETING %s:%s" % (path, record))
         self._database.delete(record)
         self.get_new_records()
         self.row_deleted(path)
@@ -299,9 +321,12 @@ class jppyModel(gtk.GenericTreeModel):
     def set_value(self, iter, column, value):
         pytype = self.get_python_column_type(column)
         if pytype == types.IntType and value == "":
-            print "Damn GTK SpinButtons..."
+            self.env.log.debug("Damn GTK SpinButtons...")
             return
-        print "set_value", repr(value), pytype, column, self.getNameOfColumn(column)
+        self.env.log.debug("set_value value: %s type: %s column: %s column name: %s" % (repr(value),
+                                                                                        pytype,
+                                                                                        column,
+                                                                                        self.getNameOfColumn(column)))
         value = self.convert_type(value, pytype)
         if column == 0:
             self.records[self.get_path(iter)[0]] = value
@@ -311,10 +336,11 @@ class jppyModel(gtk.GenericTreeModel):
             attribute = self.getNameOfColumn(column)
             if attribute in contact.keys():
                 if not contact[attribute] == value:
-                    print "%s is not %s, updating contact attribute %s" % (
-                        repr(contact[attribute]), repr(value), attribute)
+                    #self.env.log.debug("%s is not %s, updating contact attribute %s" % (repr(contact[attribute]),
+                    #                                                                    repr(value),
+                    #                                                                    attribute))
                     contact[attribute] = value
-                    print "Changed to %s" % contact[attribute]
+                    #self.env.log.debug("Changed to %s" % contact[attribute])
             elif hasattr(contact, attribute):
                 if not getattr(contact,attribute) == value:
                     setattr(contact, attribute, value)
@@ -360,14 +386,13 @@ class jppyModel(gtk.GenericTreeModel):
 
 class ContactsModel(jppyModel):
     sort_key = ("lastname",)
-    def __init__(self):
+    def __init__(self, env):
         # jppy attribute name, human label, glade widget name
         # order is from contact_as_list_item() in pytype_contacts.c
         
         # hmm.. this is quite nasty.
 
-        self._database = jppy.addressBook()
-        self.newrecord = jppy.Contact
+        self._database = env.addressBook
 
         im    = self._database.getIMLabels()
         types = self._database.getAddressTypeLabels()        
@@ -455,11 +480,11 @@ class ContactsModel(jppyModel):
                          ("showphone","Default Phone Label","Showphone",genum,pyenum),
                          ("currentphone","Default Phone","Currentphone",gstring,pystring),
 
-                         ("category","Category","ContactsCategoryOptionMenu",genum,pyenum),
+                         ("category","Category","CategoryComboBox",genum,pyenum),
                          ("secret","Private","ContactsPrivateCheckButton",gbool,pybool),
                          ]
         
-	jppyModel.__init__(self)
+	jppyModel.__init__(self, env)
 
     def getListOfNamesForColumn(self,column,node):
         if column == 54:
@@ -476,12 +501,11 @@ class TasksModel(jppyModel):
     filter = {'hide_completed': True}
     sort_key = ("description",)
         
-    def __init__(self):
+    def __init__(self, env):
         # jpilot attribute name, human label, glade widget name
         # order is from todo_as_list_item() in pytype_todos.c
-        self._database = jppy.taskList()
+        self._database = env.taskList
 
-        self.newrecord = jppy.Todo
         cats = self._database.getCategories()
 
         self._labels = {6: cats}
@@ -492,14 +516,14 @@ class TasksModel(jppyModel):
                          ("priority","Priority","TasksPriority",genum, pyenum),
                          ("note","Note","TasksNote",gstring,pystring),                         
 
-                         ("category","Category","TasksCategoryOptionMenu",genum,pyenum),
+                         ("category","Category","CategoryComboBox",genum,pyenum),
                          ("secret","Private","ContactsPrivateCheckButton",gbool, pybool)
                          ]
 
-        jppyModel.__init__(self)
+        jppyModel.__init__(self, env)
     
     def get_new_records(self):
-        print self.filter
+        self.env.log.debug(self.filter)
         self.records = []
         self._records = self._database.records(search=self.search,
                                                categories=self.categories)
@@ -511,44 +535,44 @@ class TasksModel(jppyModel):
         if self.queryfilter:
             self.records = queryLanguage.runQuery(self.queryfilter, self.records)
             if not isinstance(self.records, types.ListType):
-                print "WARNING: %s" % self.records
+                self.env.log.warn("WARNING: %s" % self.records)
                 self.records = []
         self.records = self.collate(self.records)
 
 class MemosModel(jppyModel):
     sort_key = ("text",)
-    def __init__(self):
-        self._database = jppy.memoList()
-        self.newrecord = jppy.Memo        
+    def __init__(self, env):
+        self._database = env.memoList
+
         cats = self._database.getCategories()
         self._labels = {2: cats}
         
         
         # jpilot attribute name, human label, glade widget name
         self._columns = [("text","Memo","Description",gstring,pystring),
-                         ("category","Category","CategoryOptionMenu",genum,pyenum),
+                         ("category","Category","CategoryComboBox",genum,pyenum),
                          ("secret","Private","PrivateCheckButton",gbool, pybool)]
 
-        jppyModel.__init__(self)
+        jppyModel.__init__(self, env)
         
     def shouldListViewShowOneLine(self):
         return True
 
 class Memos32Model(jppyModel):
     sort_key = ("text",)
-    def __init__(self):
-        self._database = jppy.memo32List()
-        self.newrecord = jppy.Memo        
+    def __init__(self, env):
+        self._database = env.memo32List
+
         cats = self._database.getCategories()
         self._labels = {2: cats}
         
         
         # jpilot attribute name, human label, glade widget name
         self._columns = [("text","Memo","Description",gstring,pystring),
-                         ("category","Category","CategoryOptionMenu",genum,pyenum),
+                         ("category","Category","CategoryComboBox",genum,pyenum),
                          ("secret","Private","PrivateCheckButton",gbool, pybool)]
 
-        jppyModel.__init__(self)
+        jppyModel.__init__(self, env)
         
     def shouldListViewShowOneLine(self):
         return True
