@@ -1,7 +1,14 @@
+#include <stddef.h>
 #include <pi-contact.h>
+#include <pi-address.h>
 #include <pi-todo.h>
 #include <pi-memo.h>
 #include <pi-datebook.h>
+#include <pi-buffer.h>
+
+#include "pytype_basics.h"
+
+void new_Address(struct Address *a);
 
 void new_Contact(struct Contact *a);
 
