@@ -4,7 +4,7 @@
 
 extern PyObject* PyPiAddress_New();
 extern PyObject* PyPiAddress_Wrap(struct Address*, PCRecType, unsigned int, unsigned char,
-				 int, void*);
+				 int, void*, PyObject *);
 
 extern PyTypeObject AddressType;
 
