@@ -833,6 +833,7 @@ int PyPiContact_SetItem(PyPiContact* self, PyObject* key, PyObject* value) {
   char buf[255];
   char *keystring;
   PyObject *pytup, *pyint, *pystr;
+  Py_ssize_t picture_length;
   struct ContactPicture picture;
   int n;
 
@@ -954,7 +955,8 @@ int PyPiContact_SetItem(PyPiContact* self, PyObject* key, PyObject* value) {
       Py_DECREF(key);
       return -1;
     }
-    PyString_AsStringAndSize(value, (char **)(&picture.data), &picture.length);
+    PyString_AsStringAndSize(value, (char **)(&picture.data), &picture_length);
+    picture.length = Py_SAFE_DOWNCAST(picture_length, Py_ssize_t, int);
     Contact_add_picture(&(self->a), &picture);
     // find the picture data again :-/
     for(n=0;n<MAX_CONTACT_BLOBS;n++) {
