@@ -4,7 +4,7 @@
 
 extern PyObject* PyPiMemo_New();
 extern PyObject* PyPiMemo_Wrap(struct Memo*, PCRecType, unsigned int, unsigned char,
-				 int, void*);
+				 int, void*, PyObject *);
 
 extern PyTypeObject MemoType;
 
